# (C) Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# stdlib
from collections import defaultdict
import itertools
import re
import socket
import time
import xmlrpclib

# 3p
import supervisor.xmlrpc

# project
import monasca_agent.collector.checks as checks

DEFAULT_HOST = 'localhost'
DEFAULT_PORT = '9001'
DEFAULT_SOCKET_IP = 'http://127.0.0.1'

STATUS = {
    'STOPPED': 'CRITICAL',
    'STARTING': 'UNKNOWN',
    'RUNNING': 'OK',
    'BACKOFF': 'CRITICAL',
    'STOPPING': 'CRITICAL',
    'EXITED': 'CRITICAL',
    'FATAL': 'CRITICAL',
    'UNKNOWN': 'UNKNOWN'
}

PROCESS_STATUS = {
    'CRITICAL': 'down',
    'OK': 'up',
    'UNKNOWN': 'unknown'
}

PROCESS_STATE = {
    'CRITICAL': 0,
    'OK': 1,
    'UNKNOWN': -1
}

SERVER_STATE = {
    'DOWN': 1,
    'UP': 0
}

SERVER_TAG = 'supervisord_server'

PROCESS_TAG = 'supervisord_process'


def _format_time(x):
    return time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(x))

SERVER_SERVICE_CHECK = 'supervisord.can_connect'
PROCESS_SERVICE_CHECK = 'supervisord.process.status'
PROCESS_UP_TIME_CHECK = 'supervisord.process.uptime'
PROCESS_COUNT_UP_CHECK = 'supervisord.process.count.status_up'
PROCESS_COUNT_DOWN_CHECK = 'supervisord.process.count.status_down'
PROCESS_COUNT_UNKNOWN_CHECK = 'supervisord.process.count.status_unknown'


class Supervisord(checks.AgentCheck):

    def check(self, instance):
        server_name = instance.get('name')
        proc_details_check = instance.get('proc_details_check', True)
        if proc_details_check in ['False', 'false']:
            proc_details_check = False
        proc_uptime_check = instance.get('proc_uptime_check', True)
        if proc_uptime_check in ['False', 'false']:
            proc_uptime_check = False

        if not server_name or not server_name.strip():
            raise Exception("Supervisor server name not specified in yaml configuration.")

        dimensions = self._set_dimensions({'server_name': server_name}, instance)
        supe = self._connect(instance)
        count_by_status = defaultdict(int)

        # Gather all process information
        try:
            processes = supe.getAllProcessInfo()
        except xmlrpclib.Fault as error:
            raise Exception(
                'An error occurred while reading process information: %s %s'
                % (error.faultCode, error.faultString)
            )
        except socket.error as error:
            host = instance.get('host', DEFAULT_HOST)
            port = instance.get('port', DEFAULT_PORT)
            sock = instance.get('socket')
            if sock is None:
                msg = 'Cannot connect to http://%s:%s. ' \
                      'Make sure supervisor is running and XML-RPC ' \
                      'inet interface is enabled.' % (host, port)
            else:
                msg = 'Cannot connect to %s. Make sure sure supervisor ' \
                      'is running and socket is enabled and socket file' \
                      ' has the right permissions.' % sock

            server_details = {'server_details': msg}
            self.gauge(SERVER_SERVICE_CHECK, SERVER_STATE['DOWN'],
                       dimensions=dimensions, value_meta=server_details)
            raise Exception(msg)

        except xmlrpclib.ProtocolError as error:
            if error.errcode == 401:  # authorization error
                msg = 'Username or password to %s are incorrect.' % server_name
            else:
                msg = "An error occurred while connecting to %s: "\
                    "%s %s " % (server_name, error.errcode, error.errmsg)

            server_details = {'server_details': msg}
            self.gauge(SERVER_SERVICE_CHECK, SERVER_STATE['DOWN'],
                       dimensions=dimensions, value_meta=server_details)
            raise Exception(msg)

        # If we're here, we were able to connect to the server
        self.gauge(SERVER_SERVICE_CHECK, SERVER_STATE['UP'], dimensions=dimensions)

        # Filter monitored processes on configuration directives
        proc_regex = instance.get('proc_regex', [])
        if not isinstance(proc_regex, list):
            raise Exception("Invalid proc_regex.")

        proc_names = instance.get('proc_names', [])
        if not isinstance(proc_names, list):
            raise Exception("Invalid proc_names.")

        # Collect information on each monitored process
        monitored_processes = []

        # monitor all processes if no filters were specified
        if len(proc_regex) == 0 and len(proc_names) == 0:
            monitored_processes = processes

        for pattern, process in itertools.product(proc_regex, processes):
            try:
                if re.match(pattern, process['name']) and process not in monitored_processes:
                    monitored_processes.append(process)
            except re.error:
                raise

        for process in processes:
            if process['name'] in proc_names and process not in monitored_processes:
                monitored_processes.append(process)

        # Report service checks and uptime for each process
        for proc in monitored_processes:
            proc_name = proc['name']
            dimensions[PROCESS_TAG] = proc_name

            # Retrieve status and update status count
            status = STATUS[proc['statename']]
            count_by_status[status] += 1

            # Report process details
            if proc_details_check:
                msg = self._build_message(proc)
                self.log.info('process details: %s' % msg)
                self.gauge(PROCESS_SERVICE_CHECK, PROCESS_STATE[status],
                           dimensions=dimensions)

            # Report Uptime
            if proc_uptime_check:
                uptime = self._extract_uptime(proc)
                self.gauge(PROCESS_UP_TIME_CHECK, uptime, dimensions=dimensions)

            dimensions.pop(PROCESS_TAG, None)

        # Report counts by status
        self.gauge(PROCESS_COUNT_UP_CHECK, count_by_status['OK'],
                   dimensions=dimensions)
        self.gauge(PROCESS_COUNT_DOWN_CHECK, count_by_status['CRITICAL'],
                   dimensions=dimensions)
        self.gauge(PROCESS_COUNT_UNKNOWN_CHECK, count_by_status['UNKNOWN'],
                   dimensions=dimensions)

    @staticmethod
    def _connect(instance):
        sock = instance.get('socket')
        if sock is not None:
            host = instance.get('host', DEFAULT_SOCKET_IP)
            transport = supervisor.xmlrpc.SupervisorTransport(None, None, sock)
            server = xmlrpclib.ServerProxy(host, transport=transport)
        else:
            host = instance.get('host', DEFAULT_HOST)
            port = instance.get('port', DEFAULT_PORT)
            user = instance.get('user')
            password = instance.get('pass')
            auth = '%s:%s@' % (user, password) if user and password else ''
            server = xmlrpclib.Server('http://%s%s:%s/RPC2' % (auth, host, port))
        return server.supervisor

    @staticmethod
    def _extract_uptime(proc):
        start, now = int(proc['start']), int(proc['now'])
        status = proc['statename']
        active_state = status in ['BACKOFF', 'RUNNING', 'STOPPING']
        return now - start if active_state else 0

    @staticmethod
    def _build_message(proc):
        start, stop, now = int(proc['start']), int(proc['stop']), int(proc['now'])
        proc['now_str'] = _format_time(now)
        proc['start_str'] = _format_time(start)
        proc['stop_str'] = '' if stop == 0 else _format_time(stop)

        return """Current time: %(now_str)s
Process name: %(name)s
Process group: %(group)s
Description: %(description)s
Error log file: %(stderr_logfile)s
Stdout log file: %(stdout_logfile)s
Log file: %(logfile)s
State: %(statename)s
Start time: %(start_str)s
Stop time: %(stop_str)s
Exit Status: %(exitstatus)s""" % proc



# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP

import logging
import os
import yaml

from monasca_setup import agent_config
from monasca_setup.detection import Plugin

log = logging.getLogger(__name__)


class System(Plugin):

    """No configuration here, the system metrics are assumed so this is either on or off.

    """
    system_metrics = ['network', 'disk', 'load', 'memory', 'cpu']

    def _detect(self):
        """Run detection, set self.available True if the service is detected.

        """
        self.available = True

    def build_config(self):
        """Build the configs for the system metrics as Plugin objects and return.

        """
        config = agent_config.Plugins()
        for metric in System.system_metrics:
            try:
                with open(os.path.join(self.template_dir, 'conf.d/' + metric + '.yaml'), 'r') as metric_template:
                    default_config = yaml.safe_load(metric_template.read())
                config[metric] = default_config
                if self.args:
                    for arg in self.args:
                        config[metric]['instances'][0][arg] = self.literal_eval(self.args[arg])
                log.info('\tConfigured {0}'.format(metric))
            except (OSError, IOError):
                log.info('\tUnable to configure {0}'.format(metric))
                continue

        return config

    def dependencies_installed(self):
        return True



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import logging
import six

from monascaclient import ksclient

import monasca_agent.common.singleton as singleton

log = logging.getLogger(__name__)


# Make this a singleton class so we don't get the token every time
# the class is created
@six.add_metaclass(singleton.Singleton)
class Keystone(object):

    def __init__(self, config):
        self.config = config
        self._keystone_client = None
        self._token = None

    def get_credential_args(self):
        auth_url = self.config.get('keystone_url', None)
        username = self.config.get('username', None)
        password = str(self.config.get('password', None))
        user_domain_id = self.config.get('user_domain_id', None)
        user_domain_name = self.config.get('user_domain_name', None)
        insecure = self.config.get('insecure', False)
        cacert = self.config.get('ca_file', None)
        project_id = self.config.get('project_id', None)
        project_name = self.config.get('project_name', None)
        project_domain_name = self.config.get('project_domain_name', None)
        project_domain_id = self.config.get('project_domain_id', None)
        keystone_timeout = self.config.get('keystone_timeout', None)

        kc_args = {'auth_url': auth_url,
                   'username': username,
                   'password': password,
                   'keystone_timeout': keystone_timeout}

        if user_domain_id:
            kc_args.update({'user_domain_id': user_domain_id})
        elif user_domain_name:
            kc_args.update({'user_domain_name': user_domain_name})

        if insecure:
            kc_args.update({'insecure': insecure})
        else:
            if cacert:
                kc_args.update({'os_cacert': cacert})
        if project_id:
            kc_args.update({'project_id': project_id})
        elif project_name:
            kc_args.update({'project_name': project_name})
            if project_domain_name:
                kc_args.update({'domain_name': project_domain_name})
            if project_domain_id:
                kc_args.update({'domain_id': project_domain_id})
        return kc_args

    def _get_ksclient(self):
        """Get a new keystone client object.

        The client provides a monasca_url property whose value is pulled from
        Keystone Service Catalog filtering by service_type, endpoint_type
        and/or region_name.

        """
        service_type = self.config.get('service_type', None)
        endpoint_type = self.config.get('endpoint_type', None)
        region_name = self.config.get('region_name', None)

        kc_args = self.get_credential_args()
        if service_type:
            kc_args.update({'service_type': service_type})
        if endpoint_type:
            kc_args.update({'endpoint_type': endpoint_type})
        if region_name:
            kc_args.update({'region_name': region_name})

        return ksclient.KSClient(**kc_args)

    def get_monasca_url(self):
        if not self._keystone_client:
            self.get_token()

        if self._keystone_client:
            return self._keystone_client.monasca_url
        else:
            return None

    def get_token(self):
        """Validate token is project scoped and return it if it is

        project_id and auth_token were fetched when keystone client was created

        """
        if not self._token:
            if not self._keystone_client:
                try:
                    self._keystone_client = self._get_ksclient()
                except Exception as exc:
                    log.error("Unable to create the Keystone Client. " +
                              "Error was {0}".format(repr(exc)))
                    return None

            self._token = self._keystone_client.token

        return self._token

    def refresh_token(self):
        """Gets a new keystone client object and token

        This method should be called if the token has expired

        """
        self._token = None
        self._keystone_client = None
        return self.get_token()



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP


class Infinity(Exception):
    pass


class UnknownValue(Exception):
    pass


class CheckException(Exception):
    pass


class NaN(CheckException):
    pass


class PathNotFound(Exception):
    pass



#!/usr/bin/env python
# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP

""" Detect running daemons then configure and start the agent.
"""

import argparse
from glob import glob
import json
import logging
import os
import pwd
import socket
import subprocess
import sys

import agent_config
import monasca_setup.utils as utils
from monasca_setup.utils import write_template
from service.detection import detect_init

log = logging.getLogger(__name__)

CUSTOM_PLUGIN_PATH = '/usr/lib/monasca/agent/custom_detect.d'
# dirname is called twice to get the dir 1 above the location of the script
PREFIX_DIR = os.path.dirname(os.path.dirname(os.path.realpath(sys.argv[0])))


def main(argv=None):
    parser = argparse.ArgumentParser(description='Configure and setup the agent. In a full run it will detect running' +
                                                 ' daemons then configure and start the agent.',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    args = parse_arguments(parser)

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG, format="%(levelname)s: %(message)s")
    else:
        logging.basicConfig(level=logging.INFO, format="%(levelname)s: %(message)s")

    if args.dry_run:
        log.info("Running in dry run mode, no changes will be made only reported")

    # Detect and if possibly enable the agent service
    agent_service = detect_init(PREFIX_DIR, args.config_dir, args.log_dir, args.template_dir, username=args.user)

    # Skip base setup if only installing plugins or running specific detection
    # plugins
    if not args.install_plugins_only and args.detection_plugins is None:
        if not args.skip_enable:
            agent_service.enable()

        # Verify required options
        if args.username is None or args.password is None or args.keystone_url is None:
            log.error('Username, password and keystone_url are required when running full configuration.')
            parser.print_help()
            sys.exit(1)
        base_configuration(args)

    # Collect the set of detection plugins to run
    detected_plugins = utils.discover_plugins(CUSTOM_PLUGIN_PATH)
    if args.system_only:
        from detection.plugins.system import System
        plugins = [System]
    elif args.detection_plugins is not None:
        plugins = utils.select_plugins(args.detection_plugins, detected_plugins)
    elif args.skip_detection_plugins is not None:
        plugins = utils.select_plugins(args.skip_detection_plugins, detected_plugins, skip=True)
    else:
        plugins = detected_plugins
    plugin_names = [p.__name__ for p in plugins]

    if args.remove:  # Remove entries for each plugin from the various plugin config files
        changes = remove_config(args, plugin_names)
    else:
        # Run detection for all the plugins, halting on any failures if plugins were specified in the arguments
        detected_config = plugin_detection(plugins, args.template_dir, args.detection_args,
                                           args.detection_args_json,
                                           skip_failed=(args.detection_plugins is None))
        if detected_config is None:
            return 1  # Indicates detection problem, skip remaining steps and give non-zero exit code

        changes = modify_config(args, detected_config)

    # Don't restart if only doing detection plugins and no changes found
    if args.detection_plugins is not None and not changes:
        log.info('No changes found for plugins {0}, skipping restart of Monasca Agent'.format(plugin_names))
        return 0
    elif args.dry_run:
        log.info('Running in dry mode, skipping changes and restart of Monasca Agent')
        return 0

    # Now that the config is built, start the service
    if args.install_plugins_only:
        log.info('Command line option install_plugins_only set, skipping '
                 'service (re)start.')
    else:
        try:
            agent_service.start(restart=True)
        except subprocess.CalledProcessError:
            log.error('The service did not startup correctly see %s' % args.log_dir)


def base_configuration(args):
    """Write out the primary Agent configuration and setup the service.
    :param args: Arguments from the command line
    :return: None
    """
    stat = pwd.getpwnam(args.user)

    uid = stat.pw_uid
    gid = stat.pw_gid

    # Write the main agent.yaml - Note this is always overwritten
    log.info('Configuring base Agent settings.')
    dimensions = {}
    # Join service in with the dimensions
    if args.service:
        dimensions.update({'service': args.service})
    if args.dimensions:
        dimensions.update(dict(item.strip().split(":") for item in args.dimensions.split(",")))

    args.dimensions = dict((name, value) for (name, value) in dimensions.iteritems())
    write_template(os.path.join(args.template_dir, 'agent.yaml.template'),
                   os.path.join(args.config_dir, 'agent.yaml'),
                   {'args': args, 'hostname': socket.getfqdn()},
                   group=gid,
                   user=uid,
                   is_yaml=True)

    # Write the supervisor.conf
    write_template(os.path.join(args.template_dir, 'supervisor.conf.template'),
                   os.path.join(args.config_dir, 'supervisor.conf'),
                   {'prefix': PREFIX_DIR, 'log_dir': args.log_dir, 'monasca_user': args.user},
                   user=uid,
                   group=gid)


def modify_config(args, detected_config):
    """Compare existing and detected config for each check plugin and write out
       the plugin config if there are changes
    """
    modified_config = False

    for detection_plugin_name, new_config in detected_config.iteritems():
        if args.overwrite:
            modified_config = True
            if args.dry_run:
                continue
            else:
                agent_config.save_plugin_config(args.config_dir, detection_plugin_name, args.user, new_config)
        else:
            config = agent_config.read_plugin_config_from_disk(args.config_dir, detection_plugin_name)
            # merge old and new config, new has precedence
            if config is not None:
                # For HttpCheck, if the new input url has the same host and
                # port but a different protocol comparing with one of the
                # existing instances in http_check.yaml, we want to keep the
                # existing http check instance and replace the url with the
                # new protocol. If name in this instance is the same as the
                # url, we replace name with new url too.
                # For more details please see:
                # monasca-agent/docs/DeveloperDocs/agent_internals.md
                if detection_plugin_name == "http_check":
                    # Save old http_check urls from config for later comparison
                    config_urls = [i['url'] for i in config['instances'] if
                                   'url' in i]

                    # Check endpoint change, use new protocol instead
                    # Note: config is possibly changed after running
                    # check_endpoint_changes function.
                    config = agent_config.check_endpoint_changes(new_config, config)

                agent_config.merge_by_name(new_config['instances'], config['instances'])
                # Sort before compare, if instances have no name the sort will
                #  fail making order changes significant
                try:
                    new_config['instances'].sort(key=lambda k: k['name'])
                    config['instances'].sort(key=lambda k: k['name'])
                except Exception:
                    pass

                if detection_plugin_name == "http_check":
                    new_config_urls = [i['url'] for i in new_config['instances'] if 'url' in i]
                    # Don't write config if no change
                    if new_config_urls == config_urls and new_config == config:
                        continue
                else:
                    if new_config == config:
                        continue
            modified_config = True
            if args.dry_run:
                log.info("Changes would be made to the config file for the {0}"
                         " check plugin".format(detection_plugin_name))
            else:
                agent_config.save_plugin_config(args.config_dir, detection_plugin_name, args.user, new_config)
    return modified_config


def validate_positive(value):
    int_value = int(value)
    if int_value <= 0:
        raise argparse.ArgumentTypeError("%s must be greater than zero" % value)
    return int_value


def parse_arguments(parser):
    parser.add_argument(
        '-u', '--username', help="Username used for keystone authentication. Required for basic configuration.")
    parser.add_argument(
        '-p', '--password', help="Password used for keystone authentication. Required for basic configuration.")
    parser.add_argument('--user_domain_id', help="User domain id for keystone authentication", default='')
    parser.add_argument('--user_domain_name', help="User domain name for keystone authentication", default='')
    parser.add_argument('--keystone_url', help="Keystone url. Required for basic configuration.")
    parser.add_argument('--project_name', help="Project name for keystone authentication", default='')
    parser.add_argument('--project_domain_id', help="Project domain id for keystone authentication", default='')
    parser.add_argument('--project_domain_name', help="Project domain name for keystone authentication", default='')
    parser.add_argument('--project_id', help="Keystone project id  for keystone authentication", default='')
    parser.add_argument('--monasca_url', help="Monasca API url, if not defined the url is pulled from keystone",
                        default='')
    parser.add_argument('--service_type', help="Monasca API url service type in keystone catalog", default='')
    parser.add_argument('--endpoint_type', help="Monasca API url endpoint type in keystone catalog", default='')
    parser.add_argument('--region_name', help="Monasca API url region name in keystone catalog", default='')
    parser.add_argument('--system_only', help="Setup the service but only configure the base config and system " +
                                              "metrics (cpu, disk, load, memory, network).",
                        action="store_true", default=False)
    parser.add_argument('-d', '--detection_plugins', nargs='*',
                        help="Skip base config and service setup and only configure this space separated list. " +
                             "This assumes the base config has already run.")
    parser.add_argument('--skip_detection_plugins', nargs='*',
                        help="Skip detection for all plugins in this space separated list.")
    detection_args_group = parser.add_mutually_exclusive_group()
    detection_args_group.add_argument('-a', '--detection_args', help="A string of arguments that will be passed to detection " +
                                      "plugins. Only certain detection plugins use arguments.")
    detection_args_group.add_argument('-json', '--detection_args_json',
                                      help="A JSON string that will be passed to detection plugins that parse JSON.")
    parser.add_argument('--check_frequency', help="How often to run metric collection in seconds",
                        type=validate_positive, default=30)
    parser.add_argument('--num_collector_threads', help="Number of Threads to use in Collector " +
                                                        "for running checks", type=validate_positive, default=1)
    parser.add_argument('--pool_full_max_retries', help="Maximum number of collection cycles where all of the threads " +
                                                        "in the pool are still running plugins before the " +
                                                        "collector will exit and be restart",
                        type=validate_positive, default=4)
    parser.add_argument('--plugin_collect_time_warn', help="Number of seconds a plugin collection time exceeds " +
                                                           "that causes a warning to be logged for that plugin",
                        type=validate_positive, default=6)
    parser.add_argument('--dimensions', help="Additional dimensions to set for all metrics. A comma separated list " +
                                             "of name/value pairs, 'name:value,name2:value2'")
    parser.add_argument('--ca_file', help="Sets the path to the ca certs file if using certificates. " +
                                          "Required only if insecure is set to False", default='')
    parser.add_argument('--insecure', help="Set whether certificates are used for Keystone authentication",
                        default=False)
    parser.add_argument('--config_dir', help="Configuration directory", default='/etc/monasca/agent')
    parser.add_argument('--log_dir', help="monasca-agent log directory", default='/var/log/monasca/agent')
    parser.add_argument('--log_level', help="monasca-agent logging level (ERROR, WARNING, INFO, DEBUG)", required=False,
                        default='WARN')
    parser.add_argument('--template_dir', help="Alternative template directory",
                        default=os.path.join(PREFIX_DIR, 'share/monasca/agent'))
    parser.add_argument('--overwrite',
                        help="Overwrite existing plugin configuration. " +
                             "The default is to merge. agent.yaml is always overwritten.",
                        action="store_true")
    parser.add_argument('-r', '--remove', help="Rather than add the detected configuration remove it.",
                        action="store_true", default=False)
    parser.add_argument('--skip_enable', help="By default the service is enabled, " +
                                              "which requires the script run as root. Set this to skip that step.",
                        action="store_true")
    parser.add_argument('--install_plugins_only', help="Only update plugin "
                        "configuration, do not configure services, users, etc."
                        " or restart services",
                        action="store_true")
    parser.add_argument('--user', help="User name to run monasca-agent as", default='mon-agent')
    parser.add_argument('-s', '--service', help="Service this node is associated with, added as a dimension.")
    parser.add_argument('--amplifier', help="Integer for the number of additional measurements to create. " +
                                            "Additional measurements contain the 'amplifier' dimension. " +
                                            "Useful for load testing; not for production use.", default=0)
    parser.add_argument('-v', '--verbose', help="Verbose Output", action="store_true")
    parser.add_argument('--dry_run', help="Make no changes just report on changes", action="store_true")
    parser.add_argument('--max_buffer_size',
                        help="Maximum number of batches of measurements to"
                             " buffer while unable to communicate with monasca-api",
                        default=1000)
    parser.add_argument('--max_measurement_buffer_size',
                        help="Maximum number of measurements to buffer when unable to communicate"
                             " with the monasca-api",
                        default=-1)
    parser.add_argument('--backlog_send_rate',
                        help="Maximum number of buffered batches of measurements to send at"
                             " one time when connection to the monasca-api is restored",
                        default=1000)
    parser.add_argument('--monasca_statsd_port',
                        help="Statsd daemon port number",
                        default=8125)
    return parser.parse_args()


def plugin_detection(plugins, template_dir, detection_args, detection_args_json, skip_failed=True, remove=False):
    """Runs the detection step for each plugin in the list and returns the complete detected agent config.
    :param plugins: A list of detection plugin classes
    :param template_dir: Location of plugin configuration templates
    :param detection_args: Arguments passed to each detection plugin
    :param skip_failed: When False any detection failure causes the run to halt and return None
    :return: An agent_config instance representing the total configuration from all detection plugins run.
    """
    plugin_config = agent_config.Plugins()
    if detection_args_json:
        json_data = json.loads(detection_args_json)
    for detect_class in plugins:
        # todo add option to install dependencies
        if detection_args_json:
            detect = detect_class(template_dir, False, **json_data)
        else:
            detect = detect_class(template_dir, False, detection_args)
        if detect.available:
            new_config = detect.build_config_with_name()
            if not remove:
                log.info('Configuring {0}'.format(detect.name))
            if new_config is not None:
                plugin_config.merge(new_config)
        elif not skip_failed:
            log.warn('Failed detection of plugin {0}.'.format(detect.name) +
                     "\n\tPossible causes: Service not found or missing arguments.")
            return None

    return plugin_config


def remove_config(args, plugin_names):
    """Parse all configuration removing any configuration built by plugins in plugin_names
       Note there is no concept of overwrite for removal.
    :param args: specified arguments
    :param plugin_names: A list of the plugin names to remove from the config
    :return: True if changes, false otherwise
    """
    changes = False
    existing_config_files = glob(os.path.join(args.config_dir, 'conf.d', '*.yaml'))
    detected_plugins = utils.discover_plugins(CUSTOM_PLUGIN_PATH)
    plugins = utils.select_plugins(args.detection_plugins, detected_plugins)

    if (args.detection_args or args.detection_args_json):
        detected_config = plugin_detection(
            plugins, args.template_dir, args.detection_args, args.detection_args_json,
            skip_failed=(args.detection_plugins is None), remove=True)

    for file_path in existing_config_files:
        deletes = False
        plugin_name = os.path.splitext(os.path.basename(file_path))[0]
        config = agent_config.read_plugin_config_from_disk(args.config_dir, plugin_name)
        new_instances = []  # To avoid odd issues from iterating over a list you delete from, build a new instead
        if args.detection_args is None:
            for inst in config['instances']:
                if 'built_by' in inst and inst['built_by'] in plugin_names:
                    changes = True
                    deletes = True
                    continue
                new_instances.append(inst)
            config['instances'] = new_instances
        else:
            for detected_key in detected_config.keys():
                for inst in detected_config[detected_key]['instances']:
                    if inst in config['instances']:
                        changes = True
                        deletes = True
                        config['instances'].remove(inst)
        if deletes:
            agent_config.delete_from_config(args, config, file_path, plugin_name)
    return changes
if __name__ == "__main__":
    sys.exit(main())



import unittest
from StringIO import StringIO

from tests.common import get_check


CONFIG = """
init_config:

instances:
    - host: 127.0.0.1
      port: 2181
      dimensions: {}
"""


class TestZookeeper(unittest.TestCase):

    def test_zk_stat_parsing_lt_v344(self):
        Zookeeper, instances = get_check('zk', CONFIG)
        stat_response = """Zookeeper version: 3.2.2--1, built on 03/16/2010 07:31 GMT
Clients:
 /10.42.114.160:32634[1](queued=0,recved=12,sent=0)
 /10.37.137.74:21873[1](queued=0,recved=53613,sent=0)
 /10.37.137.74:21876[1](queued=0,recved=57436,sent=0)
 /10.115.77.32:32990[1](queued=0,recved=16,sent=0)
 /10.37.137.74:21891[1](queued=0,recved=55011,sent=0)
 /10.37.137.74:21797[1](queued=0,recved=19431,sent=0)

Latency min/avg/max: -10/0/20007
Received: 101032173
Sent: 0
Outstanding: 0
Zxid: 0x1034799c7
Mode: leader
Node count: 487
"""
        expected = [
            ('zookeeper.latency.min', -10),
            ('zookeeper.latency.avg', 0),
            ('zookeeper.latency.max', 20007),
            ('zookeeper.bytes_received', 101032173),
            ('zookeeper.bytes_sent', 0),
            ('zookeeper.connections', 6),
            ('zookeeper.bytes_outstanding', 0),
            ('zookeeper.zxid.epoch', 1),
            ('zookeeper.zxid.count', 55024071),
            ('zookeeper.nodes', 487),
        ]

        buf = StringIO(stat_response)
        metrics, dimensions = Zookeeper.parse_stat(buf)

        self.assertEqual(dimensions, {'mode': 'leader'})
        self.assertEqual(metrics, expected)

    def test_zk_stat_parsing_gte_v344(self):
        Zookeeper, instances = get_check('zk', CONFIG)
        stat_response = """Zookeeper version: 3.4.5--1, built on 03/16/2010 07:31 GMT
Clients:
 /10.42.114.160:32634[1](queued=0,recved=12,sent=0)
 /10.37.137.74:21873[1](queued=0,recved=53613,sent=0)
 /10.37.137.74:21876[1](queued=0,recved=57436,sent=0)
 /10.115.77.32:32990[1](queued=0,recved=16,sent=0)
 /10.37.137.74:21891[1](queued=0,recved=55011,sent=0)
 /10.37.137.74:21797[1](queued=0,recved=19431,sent=0)

Latency min/avg/max: -10/0/20007
Received: 101032173
Sent: 0
Connections: 1
Outstanding: 0
Zxid: 0x1034799c7
Mode: leader
Node count: 487
"""
        expected = [
            ('zookeeper.latency.min', -10),
            ('zookeeper.latency.avg', 0),
            ('zookeeper.latency.max', 20007),
            ('zookeeper.bytes_received', 101032173),
            ('zookeeper.bytes_sent', 0),
            ('zookeeper.connections', 1),
            ('zookeeper.bytes_outstanding', 0),
            ('zookeeper.zxid.epoch', 1),
            ('zookeeper.zxid.count', 55024071),
            ('zookeeper.nodes', 487),
        ]

        buf = StringIO(stat_response)
        metrics, dimensions = Zookeeper.parse_stat(buf)

        self.assertEqual(dimensions, {'mode': 'leader'})
        self.assertEqual(metrics, expected)



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import logging

from plugin import Plugin


log = logging.getLogger(__name__)


class ArgsPlugin(Plugin):
    """Base plugin for detection plugins that take arguments for configuration rather than do detection."""

    def _build_instance(self, arg_list):
        """If a value for each arg in the arg_list was specified build it into an instance dictionary. Also check for dimensions and add if they were specified.
        :param arg_list: Arguments to include
        :return: instance dictionary
        """
        instance = {}
        if 'dimensions' in self.args:
            instance['dimensions'] = dict(item.strip().split(":") for item in self.args['dimensions'].split(","))
        for arg in arg_list:
            if arg in self.args:
                instance[arg] = self.args[arg]
        return instance

    def _check_required_args(self, arg_list):
        """Check that the required args were specified
        :param arg_list: A list of arguments to verify were specified
        :return: True if the required args exist false otherwise
        """
        if self.args is None:
            return False
        for arg in arg_list:
            if arg not in self.args:
                return False
        return True

    def dependencies_installed(self):
        return True



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP
'''
Check the performance counters from IIS
'''
try:
    import wmi
except Exception:
    wmi = None

from monasca_agent.collector.checks import AgentCheck


class IIS(AgentCheck):
    METRICS = [
        ('iis.uptime', 'gauge', 'ServiceUptime'),

        # Network
        ('iis.net.bytes_sent', 'rate', 'TotalBytesSent'),
        ('iis.net.bytes_rcvd', 'rate', 'TotalBytesReceived'),
        ('iis.net.bytes_total', 'rate', 'TotalBytesTransferred'),
        ('iis.net.num_connections', 'gauge', 'CurrentConnections'),
        ('iis.net.files_sent', 'rate', 'TotalFilesSent'),
        ('iis.net.files_rcvd', 'rate', 'TotalFilesReceived'),
        ('iis.net.connection_attempts', 'rate', 'TotalConnectionAttemptsAllInstances'),

        # HTTP Methods
        ('iis.httpd_request_method.get', 'rate', 'TotalGetRequests'),
        ('iis.httpd_request_method.post', 'rate', 'TotalPostRequests'),
        ('iis.httpd_request_method.head', 'rate', 'TotalHeadRequests'),
        ('iis.httpd_request_method.put', 'rate', 'TotalPutRequests'),
        ('iis.httpd_request_method.delete', 'rate', 'TotalDeleteRequests'),
        ('iis.httpd_request_method.options', 'rate', 'TotalOptionsRequests'),
        ('iis.httpd_request_method.trace', 'rate', 'TotalTraceRequests'),

        # Errors
        ('iis.errors.not_found', 'rate', 'TotalNotFoundErrors'),
        ('iis.errors.locked', 'rate', 'TotalLockedErrors'),

        # Users
        ('iis.users.anon', 'rate', 'TotalAnonymousUsers'),
        ('iis.users.nonanon', 'rate', 'TotalNonAnonymousUsers'),

        # Requests
        ('iis.requests.cgi', 'rate', 'TotalCGIRequests'),
        ('iis.requests.isapi', 'rate', 'TotalISAPIExtensionRequests'),
    ]

    def __init__(self, name, init_config, agent_config):
        AgentCheck.__init__(self, name, init_config, agent_config)
        self.wmi_conns = {}

    def _get_wmi_conn(self, host, user, password):
        key = "%s:%s:%s" % (host, user, password)
        if key not in self.wmi_conns:
            self.wmi_conns[key] = wmi.WMI(computer=host, user=user,
                                          password=password)
        return self.wmi_conns[key]

    def check(self, instance):
        if wmi is None:
            raise Exception("Missing 'wmi' module")

        # Connect to the WMI provider
        host = instance.get('host', None)
        user = instance.get('username', None)
        password = instance.get('password', None)
        dimensions = self._set_dimensions(None, instance)
        sites = instance.get('sites', ['_Total'])
        w = self._get_wmi_conn(host, user, password)

        try:
            wmi_cls = w.Win32_PerfFormattedData_W3SVC_WebService()
            if not wmi_cls:
                raise Exception('Missing data from Win32_PerfFormattedData_W3SVC_WebService')
        except Exception:
            self.log.exception('Unable to fetch Win32_PerfFormattedData_W3SVC_WebService class')
            return

        # Iterate over every IIS site
        for iis_site in wmi_cls:
            # Skip any sites we don't specifically want.
            if iis_site.Name not in sites:
                continue

            # Tag with the site name if we're not using the aggregate
            if iis_site.Name == '_Total':
                dimensions.pop('site', None)
            else:
                dimensions.update({'site': iis_site.Name})

            for metric, mtype, wmi_val in self.METRICS:
                if not hasattr(iis_site, wmi_val):
                    self.log.warn('Unable to fetch metric %s. Missing %s in '
                                  'Win32_PerfFormattedData_W3SVC_WebService' % (metric, wmi_val))
                    continue

                # Submit the metric value with the correct type
                value = float(getattr(iis_site, wmi_val))
                metric_func = getattr(self, mtype)
                metric_func(metric, value, dimensions=dimensions)



# Copyright 2016 FUJITSU LIMITED
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import contextlib
import logging
import os
import unittest
import mock
import psutil
import json

from monasca_setup.detection.plugins import kibana

LOG = logging.getLogger(kibana.__name__)

_KIBANA_METRICS = ['heap_size',
                   'heap_used',
                   'load',
                   'req_sec',
                   'resp_time_avg',
                   'resp_time_max']


class JsonResponse(object):

    def __init__(self, data):
        self.data = data

    def json(self):
        return self.data


class PSUtilGetProc(object):
    cmdLine = ['kibana']

    def as_dict(self):
        return {'name': 'kibana',
                'cmdline': PSUtilGetProc.cmdLine}

    def cmdline(self):
        return self.cmdLine


class KibanaDetectionTest(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        with mock.patch.object(kibana.Kibana, '_detect') as mock_detect:
            self.kibana_plugin = kibana.Kibana('temp_dir')
            self.assertTrue(mock_detect.called)

    def _detect(self,
                kibana_plugin,
                config_is_file=True,
                deps_installed=True):
        kibana_plugin.available = False
        psutil_mock = PSUtilGetProc()

        process_iter_patch = mock.patch.object(psutil, 'process_iter',
                                               return_value=[psutil_mock])
        isfile_patch = mock.patch.object(os.path, 'isfile',
                                         return_value=config_is_file)
        deps_installed_patch = mock.patch.object(kibana_plugin,
                                                 'dependencies_installed',
                                                 return_value=deps_installed)

        with contextlib.nested(process_iter_patch,
                               isfile_patch,
                               deps_installed_patch) as (
                mock_process_iter, mock_isfile, mock_deps_installed):
            kibana_plugin._detect()
            self.assertTrue(mock_process_iter.called)
            self.assertTrue(mock_isfile.called)
            self.assertTrue(mock_deps_installed.called)

    def _verify_kibana_conf(self, kibana_check, kibana_url):

        self.assertIn('init_config', kibana_check)
        self.assertIsNotNone(kibana_check['init_config'])
        self.assertIn('url', kibana_check['init_config'])
        self.assertEqual(kibana_check['init_config']['url'], kibana_url)

        self.assertIn('instances', kibana_check)
        self.assertEqual(1, len(kibana_check['instances']))

        for instance in kibana_check['instances']:
            self.assertIn('metrics', instance)
            self.assertEqual(list, type(instance['metrics']))
            self.assertItemsEqual(_KIBANA_METRICS, instance['metrics'])

    def _verify_process_conf(self, process_check, kibana_user):
        # minimize check here, do not check how process should work
        # just find the user

        self.assertIn('instances', process_check)
        self.assertEqual(1, len(process_check['instances']))

        for instance in process_check['instances']:
            if not kibana_user:
                self.assertNotIn('username', instance)
            else:
                self.assertIn('username', instance)
                self.assertEqual(kibana_user, instance['username'])

    def test_no_detect_no_process(self):
        with mock.patch.object(LOG, 'error') as mock_log_error:
            PSUtilGetProc.cmdLine = []
            self._detect(self.kibana_plugin)
            self.assertFalse(self.kibana_plugin.available)

            self.assertEqual(mock_log_error.call_count, 1)
            self.assertEqual(mock_log_error.call_args[0][0],
                             'Kibana process has not been found. '
                             'Plugin for Kibana will not be configured.')

    def test_no_detect_no_dependencies(self):
        with mock.patch.object(LOG, 'error') as mock_log_error:
            self._detect(self.kibana_plugin, deps_installed=False)
            self.assertFalse(self.kibana_plugin.available)

            self.assertEqual(mock_log_error.call_count, 1)
            self.assertEqual(mock_log_error.call_args[0][0],
                             'Kibana plugin dependencies are not satisfied. '
                             'Module "pyaml" not found. '
                             'Plugin for Kibana will not be configured.')

    def test_no_detect_no_default_config_file(self):
        with mock.patch.object(LOG, 'error') as mock_log_error:
            self._detect(self.kibana_plugin, config_is_file=False)
            self.assertFalse(self.kibana_plugin.available)

            self.assertEqual(mock_log_error.call_count, 1)
            self.assertEqual(mock_log_error.call_args[0][0],
                             'Kibana plugin cannot find configuration '
                             'file /opt/kibana/config/kibana.yml. '
                             'Plugin for Kibana will not be configured.')

    def test_no_detect_no_args_config_file(self):
        config_file = '/fake/config'

        patch_log_error = mock.patch.object(LOG, 'error')

        with patch_log_error as mock_log_error:
            self.kibana_plugin.args = {'kibana-config': config_file}

            self._detect(self.kibana_plugin, config_is_file=False)
            self.assertFalse(self.kibana_plugin.available)

            self.assertEqual(mock_log_error.call_count, 1)
            self.assertEqual(mock_log_error.call_args[0][0],
                             'Kibana plugin cannot find configuration '
                             'file %s. '
                             'Plugin for Kibana will not be configured.'
                             % config_file)

    def test_detect_ok(self):
        self._detect(self.kibana_plugin)
        self.assertTrue(self.kibana_plugin.available)

    def test_build_config_unreadable_config(self):
        patch_log_error = mock.patch.object(LOG, 'error')
        patch_log_exception = mock.patch.object(LOG, 'exception')
        patch_read_config = mock.patch.object(self.kibana_plugin,
                                              '_read_config',
                                              side_effect=Exception('oh'))

        with contextlib.nested(
                patch_log_error,
                patch_log_exception,
                patch_read_config
        ) as (mock_log_error, mock_log_exception, _):
            self.kibana_plugin.build_config()

            self.assertEqual(mock_log_error.call_count, 1)
            self.assertEqual(mock_log_error.call_args[0][0],
                             'Failed to read configuration at '
                             '/opt/kibana/config/kibana.yml')

            self.assertEqual(mock_log_exception.call_count, 1)
            self.assertEqual(repr(mock_log_exception.call_args[0][0]),
                             repr(Exception('oh')))

    def test_build_config_https_support(self):
        config = ('localhost', 5700, 'https')
        patch_log_error = mock.patch.object(LOG, 'error')
        patch_read_config = mock.patch.object(self.kibana_plugin,
                                              '_read_config',
                                              return_value=config)

        with contextlib.nested(
                patch_log_error,
                patch_read_config
        ) as (mock_log_error, _):
            self.assertIsNone(self.kibana_plugin.build_config())

            self.assertEqual(mock_log_error.call_count, 1)
            self.assertEqual(mock_log_error.call_args[0][0],
                             '"https" protocol is currently not supported')

    def test_build_config_no_metric_support(self):
        config = ('localhost', 5700, 'http')
        patch_log_warning = mock.patch.object(LOG, 'warning')
        patch_read_config = mock.patch.object(self.kibana_plugin,
                                              '_read_config',
                                              return_value=config)
        has_metric_patch = mock.patch.object(self.kibana_plugin,
                                             '_has_metrics_support',
                                             return_value=False)

        with contextlib.nested(
                patch_log_warning,
                patch_read_config,
                has_metric_patch
        ) as (patch_log_warning, _, __):
            self.assertIsNone(self.kibana_plugin.build_config())

            self.assertEqual(patch_log_warning.call_count, 1)
            self.assertEqual(patch_log_warning.call_args[0][0],
                             'Running kibana does not support '
                             'metrics, skipping...')

    def test_build_config_ok_no_kibana_user(self):
        self._test_build_config_ok(None)

    def test_build_config_ok_kibana_user(self):
        self._test_build_config_ok('kibana-wizard')

    def _test_build_config_ok(self, kibana_user):
        kibana_host = 'localhost'
        kibana_port = 5700
        kibana_protocol = 'http'

        kibana_cfg = (kibana_host, kibana_port, kibana_protocol)
        kibana_url = '%s://%s:%d/api/status' % (
            kibana_protocol,
            kibana_host,
            kibana_port
        )

        fixture_file = (os.path.dirname(os.path.abspath(__file__))
                        + '/../checks_d/fixtures/test_kibana.json')
        response = json.load(file(fixture_file))

        get_metric_req_ret = mock.Mock(
            wraps=JsonResponse(response)
        )

        patch_read_config = mock.patch.object(self.kibana_plugin,
                                              '_read_config',
                                              return_value=kibana_cfg)
        has_metric_patch = mock.patch.object(self.kibana_plugin,
                                             '_has_metrics_support',
                                             return_value=True)
        get_metrics_patch = mock.patch.object(self.kibana_plugin,
                                              '_get_metrics_request',
                                              return_value=get_metric_req_ret)

        self.kibana_plugin.args = {'kibana-user': kibana_user}

        with contextlib.nested(patch_read_config,
                               has_metric_patch,
                               get_metrics_patch):
            conf = self.kibana_plugin.build_config()
            self.assertIsNotNone(conf)

            self.assertItemsEqual(['kibana', 'process'], conf.keys())
            self._verify_kibana_conf(conf['kibana'], kibana_url)
            self._verify_process_conf(conf['process'], kibana_user)



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import glob
import imp
import inspect
import logging
import os
import sys
import yaml

from monasca_setup.detection import Plugin

log = logging.getLogger(__name__)


def discover_plugins(custom_path):
    """Find and import all detection plugins. It will look in detection/plugins dir of the code as well as custom_path

    :param custom_path: An additional path to search for detection plugins
    :return: A list of imported detection plugin classes.
    """

    # This was adapted from what monasca_agent.common.util.load_check_directory
    plugin_paths = glob.glob(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'detection/plugins', '*.py'))
    plugin_paths.extend(glob.glob(os.path.join(custom_path, '*.py')))

    plugins = []

    for plugin_path in plugin_paths:
        if os.path.basename(plugin_path) == '__init__.py':
            continue
        try:
            plugin = imp.load_source(os.path.splitext(os.path.basename(plugin_path))[0], plugin_path)
        except Exception:
            log.exception('Unable to import detection plugin {0}'.format(plugin_path))

        # Verify this is a subclass of Plugin
        classes = inspect.getmembers(plugin, inspect.isclass)
        for _, clsmember in classes:
            if Plugin == clsmember:
                continue
            if issubclass(clsmember, Plugin):
                plugins.append(clsmember)

    return plugins


def select_plugins(plugin_names, plugin_list, skip=False):
    """:param plugin_names: A list of names
       :param plugin_list: A list of detection plugins classes
       :param skip: Inverts the behaviour by selecting only those plugins not in plugin_names
       :return: Returns a list of plugins from plugin_list that match plugin_names
    """
    lower_plugins = [p.lower() for p in plugin_names]
    plugins = []
    for plugin in plugin_list:
        selected = (plugin.__name__.lower() in lower_plugins)
        if selected != skip:
            plugins.append(plugin)

    if len(plugins) != len(plugin_names) and not skip:
        pnames = [p.__name__ for p in plugin_list]
        log.warn("Not all plugins found, discovered plugins {0}\nAvailable plugins{1}".format(plugins,
                                                                                              pnames))

    return plugins


def write_template(template_path,
                   out_path,
                   variables,
                   group,
                   user,
                   is_yaml=False):
    """Write a file using a simple python string template.
       Assumes 640 for the permissions and root:group for ownership.

    :param template_path: Location of the Template to use
    :param out_path: Location of the file to write
    :param variables: dictionary with key/value pairs to use in writing the template
    :param group: group to set as owner of config file
    :param user: user to set as owner of config file
    :param is_yaml: is configuration file in YAML format

    :return: None
    """
    if not os.path.exists(template_path):
        print("Error no template found at {0}".format(template_path))
        sys.exit(1)
    with open(template_path, 'r') as template:
        contents = template.read().format(**variables)
        with open(out_path, 'w') as conf:
            if is_yaml:
                conf.write(yaml.safe_dump(yaml.safe_load(contents),
                                          encoding='utf-8',
                                          allow_unicode=True,
                                          default_flow_style=False))
            else:
                conf.write(contents)

    os.chown(out_path, user, group)
    os.chmod(out_path, 0o640)



# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development Company LP

import glob
import hashlib
import imp
import inspect
import itertools
import math
import optparse
import os
import platform
import re
import socket
import subprocess
import sys
import tempfile
import threading
import time
import traceback
import uuid

import logging
log = logging.getLogger(__name__)


# Tornado
try:
    from tornado import ioloop
    from tornado import version_info as tornado_version
except ImportError:
    # We are likely running the agent without the forwarder and tornado is not installed
    # Generate a warning
    log.warn('Tornado web server is not installed.  Metrics will not be forwarded to the Monasca API!')

VALID_HOSTNAME_RFC_1123_PATTERN = re.compile(
    r"^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])$")
MAX_HOSTNAME_LEN = 255
LOGGING_MAX_BYTES = 5 * 1024 * 1024

NumericTypes = (float, int, long)

import monasca_agent.common.config as configuration
from monasca_agent.common.exceptions import PathNotFound


class PidFile(object):
    """A small helper class for pidfiles. """

    PID_DIR = '/var/run/monasca-agent'

    def __init__(self, program, pid_dir=None):
        self.pid_file = "%s.pid" % program
        self.pid_dir = pid_dir or self.get_default_pid_dir()
        self.pid_path = os.path.join(self.pid_dir, self.pid_file)

    @staticmethod
    def get_default_pid_dir():
        if get_os() != 'windows':
            return PidFile.PID_DIR

        return tempfile.gettempdir()

    def get_path(self):
        # Can we write to the directory
        try:
            if os.access(self.pid_dir, os.W_OK):
                log.info("Pid file is: %s" % self.pid_path)
                return self.pid_path
        except Exception:
            log.warn("Cannot locate pid file, trying to use: %s" % tempfile.gettempdir())

        # if all else fails
        if os.access(tempfile.gettempdir(), os.W_OK):
            tmp_path = os.path.join(tempfile.gettempdir(), self.pid_file)
            log.debug("Using temporary pid file: %s" % tmp_path)
            return tmp_path
        else:
            # Can't save pid file, bail out
            log.error("Cannot save pid file anywhere")
            raise Exception("Cannot save pid file anywhere")

    def clean(self):
        try:
            path = self.get_path()
            log.debug("Cleaning up pid file %s" % path)
            os.remove(path)
            return True
        except Exception:
            log.warn("Could not clean up pid file")
            return False

    def get_pid(self):
        "Retrieve the actual pid"
        try:
            pf = open(self.get_path())
            pid_s = pf.read()
            pf.close()

            return int(pid_s.strip())
        except Exception:
            return None


class LaconicFilter(logging.Filter):
    """Filters messages, only print them once while keeping memory under control
    """
    LACONIC_MEM_LIMIT = 1024

    def __init__(self, name=""):
        logging.Filter.__init__(self, name)
        self.hashed_messages = {}

    @staticmethod
    def hash(msg):
        return hashlib.md5(msg).hexdigest()

    def filter(self, record):
        try:
            h = self.hash(record.getMessage())
            if h in self.hashed_messages:
                return 0
            else:
                # Don't blow up our memory
                if len(self.hashed_messages) >= LaconicFilter.LACONIC_MEM_LIMIT:
                    self.hashed_messages.clear()
                self.hashed_messages[h] = True
                return 1
        except Exception:
            return 1


class Timer(object):
    """Helper class """

    def __init__(self):
        self.start()

    @staticmethod
    def _now():
        return time.time()

    def start(self):
        self.started = self._now()
        self.last = self.started
        return self

    def step(self):
        now = self._now()
        step = now - self.last
        self.last = now
        return step

    def total(self, as_sec=True):
        return self._now() - self.started


class Platform(object):
    """Return information about the given platform.
    """
    @staticmethod
    def is_darwin(name=None):
        name = name or sys.platform
        return 'darwin' in name

    @staticmethod
    def is_freebsd(name=None):
        name = name or sys.platform
        return name.startswith("freebsd")

    @staticmethod
    def is_linux(name=None):
        name = name or sys.platform
        return 'linux' in name

    @staticmethod
    def is_bsd(name=None):
        """Return true if this is a BSD like operating system. """
        name = name or sys.platform
        return Platform.is_darwin(name) or Platform.is_freebsd(name)

    @staticmethod
    def is_solaris(name=None):
        name = name or sys.platform
        return name == "sunos5"

    @staticmethod
    def is_unix(name=None):
        """Return true if the platform is a unix, False otherwise. """
        name = name or sys.platform
        return (Platform.is_darwin()
                or Platform.is_linux()
                or Platform.is_freebsd()
                )

    @staticmethod
    def is_win32(name=None):
        name = name or sys.platform
        return name == "win32"


class Dimensions(object):
    """Class to update the default dimensions.
    """

    def __init__(self, agent_config):
        self.agent_config = agent_config

    def _set_dimensions(self, dimensions, instance=None):
        """Method to append the default dimensions and per instance dimensions from the config files.
        """
        new_dimensions = {'hostname': get_hostname()}

        default_dimensions = self.agent_config.get('dimensions', {})
        if default_dimensions:
            # Add or update any default dimensions that were set in the agent config file
            new_dimensions.update(default_dimensions)
        if dimensions is not None:
            # Add or update any dimensions from the plugin itself
            new_dimensions.update(dimensions.copy())
        if instance:
            # Add or update any per instance dimensions that were set in the plugin config file
            new_dimensions.update(instance.get('dimensions', {}))
        return new_dimensions


class Paths(object):
    """Return information about system paths.
    """
    def __init__(self):
        self.osname = get_os()

    def get_confd_path(self):
        bad_path = ''
        try:
            return self._unix_confd_path()
        except PathNotFound as e:
            if len(e.args) > 0:
                bad_path = e.args[0]

        cur_path = os.path.dirname(os.path.realpath(__file__))
        cur_path = os.path.join(cur_path, 'conf.d')

        if os.path.exists(cur_path):
            return cur_path

        raise PathNotFound(bad_path)

    def _unix_confd_path(self):
        try:
            return configuration.Config().get_confd_path()
        except PathNotFound:
            pass

        path = os.path.join(os.getcwd(), 'conf.d')
        if os.path.exists(path):
            return path
        raise PathNotFound(path)

    def get_checksd_path(self):
        return self._unix_checksd_path()

    def _unix_checksd_path(self):
        # Unix only will look up based on the current directory
        # because checks_d will hang with the other python modules
        cur_path = os.path.dirname(os.path.realpath(__file__))
        checksd_path = os.path.join(cur_path, '../collector/checks_d')

        if os.path.exists(checksd_path):
            return checksd_path
        raise PathNotFound(checksd_path)


def plural(count):
    if count == 1:
        return ""
    return "s"


def get_tornado_ioloop():
    if tornado_version[0] == 3:
        return ioloop.IOLoop.current()
    else:
        return ioloop.IOLoop.instance()


def get_uuid():
    # Generate a unique name that will stay constant between
    # invocations, such as platform.node() + uuid.getnode()
    # Use uuid5, which does not depend on the clock and is
    # recommended over uuid3.
    # This is important to be able to identify a server even if
    # its drives have been wiped clean.
    # Note that this is not foolproof but we can reconcile servers
    # on the back-end if need be, based on mac addresses.
    return uuid.uuid5(uuid.NAMESPACE_DNS, platform.node() + str(uuid.getnode())).hex


def timeout_command(command, timeout, command_input=None):
    # call shell-command with timeout (in seconds) and stdinput for the command (optional)
    # returns None if timeout or the command output.
    process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
    command_timer = threading.Timer(timeout, process.kill)
    try:
        command_timer.start()
        stdout, stderr = process.communicate(input=command_input.encode() if command_input else None)
        return_code = process.returncode
        return stdout, stderr, return_code
    finally:
        command_timer.cancel()


def get_os():
    "Human-friendly OS name"
    if sys.platform == 'darwin':
        return 'mac'
    elif sys.platform.find('freebsd') != -1:
        return 'freebsd'
    elif sys.platform.find('linux') != -1:
        return 'linux'
    elif sys.platform.find('win32') != -1:
        return 'windows'
    elif sys.platform.find('sunos') != -1:
        return 'solaris'
    else:
        return sys.platform


def headers(agentConfig):
    # Build the request headers
    return {
        'User-Agent': 'Mon Agent/%s' % agentConfig['version'],
        'Content-Type': 'application/x-www-form-urlencoded',
        'Accept': 'text/html, */*',
    }


def getTopIndex():
    macV = None
    if sys.platform == 'darwin':
        macV = platform.mac_ver()

    # Output from top is slightly modified on OS X 10.6 (case #28239)
    if macV and macV[0].startswith('10.6.'):
        return 6
    else:
        return 5


def isnan(val):
    if hasattr(math, 'isnan'):
        return math.isnan(val)

    # for py < 2.6, use a different check
    # http://stackoverflow.com/questions/944700/how-to-check-for-nan-in-python
    return str(val) == str(1e400 * 0)


def cast_metric_val(val):
    # ensure that the metric value is a numeric type
    if not isinstance(val, NumericTypes):
        # Try the int conversion first because want to preserve
        # whether the value is an int or a float. If neither work,
        # raise a ValueError to be handled elsewhere
        for cast in [int, float]:
            try:
                val = cast(val)
                return val
            except ValueError:
                continue
        raise ValueError
    return val


def is_valid_hostname(hostname):
    if hostname.lower() in ('localhost', 'localhost.localdomain',
                            'localhost6.localdomain6', 'ip6-localhost'):
        log.warning("Hostname: %s is local" % hostname)
        return False
    if len(hostname) > MAX_HOSTNAME_LEN:
        log.warning("Hostname: %s is too long (max length is  %s characters)" %
                    (hostname, MAX_HOSTNAME_LEN))
        return False
    if VALID_HOSTNAME_RFC_1123_PATTERN.match(hostname) is None:
        log.warning("Hostname: %s is not complying with RFC 1123" % hostname)
        return False
    return True


def get_hostname():
    """Get the canonical host name this agent should identify as. This is
       the authoritative source of the host name for the agent.

    Tries, in order:

      * agent config (agent.yaml, "hostname:")
      * 'hostname -f' (on unix)
      * socket.gethostname()
    """
    hostname = None

    # first, try the config
    config = configuration.Config()
    agent_config = config.get_config(sections='Main')
    config_hostname = agent_config.get('hostname')
    if config_hostname and is_valid_hostname(config_hostname):
        return config_hostname

    # then move on to os-specific detection
    if hostname is None:
        def _get_hostname_unix():
            try:
                # try fqdn
                p = subprocess.Popen(['/bin/hostname', '-f'], stdout=subprocess.PIPE)
                out, err = p.communicate()
                if p.returncode == 0:
                    return out.strip()
            except Exception:
                return None

        os_name = get_os()
        if os_name in ['mac', 'freebsd', 'linux', 'solaris']:
            unix_hostname = _get_hostname_unix()
            if unix_hostname and is_valid_hostname(unix_hostname):
                hostname = unix_hostname

    # fall back on socket.gethostname(), socket.getfqdn() is too unreliable
    if hostname is None:
        try:
            socket_hostname = socket.gethostname()
        except socket.error:
            socket_hostname = None
        if socket_hostname and is_valid_hostname(socket_hostname):
            hostname = socket_hostname

    if hostname is None:
        log.critical(
            'Unable to reliably determine host name. You can define one in agent.yaml or in your hosts file')
        raise Exception(
            'Unable to reliably determine host name. You can define one in agent.yaml or in your hosts file')
    else:
        return hostname


def get_parsed_args():
    parser = optparse.OptionParser()
    parser.add_option('-c', '--clean', action='store_true', default=False, dest='clean')
    parser.add_option('-v', '--verbose', action='store_true', default=False, dest='verbose',
                      help='Print out stacktraces for errors in checks')
    parser.add_option('-f', '--config-file', default=None, dest='config_file',
                      help='Location for an alternate config rather than '
                           'using the default config location.')

    try:
        options, args = parser.parse_args()
    except SystemExit:
        # Ignore parse errors
        options, args = optparse.Values({'clean': False}), []
    return options, args


def load_check_directory():
    """Return the initialized checks from checks_d, and a mapping of checks that failed to
    initialize. Only checks that have a configuration
    file in conf.d will be returned.
    """

    from monasca_agent.collector.checks import AgentCheck

    config = configuration.Config()
    agent_config = config.get_config('Main')

    initialized_checks = {}
    init_failed_checks = {}

    paths = Paths()
    checks_paths = [glob.glob(os.path.join(agent_config['additional_checksd'], '*.py'))]

    try:
        checksd_path = paths.get_checksd_path()
        checks_paths.append(glob.glob(os.path.join(checksd_path, '*.py')))
    except PathNotFound as e:
        log.error(e.args[0])
        sys.exit(3)

    try:
        confd_path = paths.get_confd_path()
    except PathNotFound as e:
        log.error(
            "No conf.d folder found at '%s' or in the directory where the Agent is currently deployed.\n" %
            e.args[0])
        sys.exit(3)

    # For backwards-compatability with old style checks, we have to load every
    # checks_d module and check for a corresponding config OR check if the old
    # config will "activate" the check.
    #
    # Once old-style checks aren't supported, we'll just read the configs and
    # import the corresponding check module
    for check in itertools.chain(*checks_paths):
        check_name = os.path.basename(check).split('.')[0]
        if check_name in initialized_checks or check_name in init_failed_checks:
            log.debug(
                'Skipping check %s because it has already been loaded from another location', check)
            continue
        try:
            check_module = imp.load_source('checksd_%s' % check_name, check)
        except Exception as e:
            traceback_message = traceback.format_exc()

            # Let's see if there is a conf.d for this check
            conf_path = os.path.join(confd_path, '%s.yaml' % check_name)
            if os.path.exists(conf_path):
                # There is a configuration file for that check but the module can't be imported
                init_failed_checks[check_name] = {'error': e, 'traceback': traceback_message}
                log.exception('Unable to import check module %s.py from checks_d' % check_name)
            else:  # There is no conf for that check. Let's not spam the logs for it.
                log.debug('Unable to import check module %s.py from checks_d' % check_name)
            continue

        check_class = None
        classes = inspect.getmembers(check_module, inspect.isclass)
        for _, clsmember in classes:
            if clsmember == AgentCheck:
                continue
            if issubclass(clsmember, AgentCheck):
                check_class = clsmember
                if AgentCheck in clsmember.__bases__:
                    continue
                else:
                    break

        if not check_class:
            if not check_name == '__init__':
                log.error('No check class (inheriting from AgentCheck) found in %s.py' % check_name)
                continue

        # Check if the config exists
        conf_path = os.path.join(confd_path, '%s.yaml' % check_name)
        if os.path.exists(conf_path):
            try:
                check_config = config.check_yaml(conf_path)
            except Exception as e:
                log.exception("Unable to parse yaml config in %s" % conf_path)
                traceback_message = traceback.format_exc()
                init_failed_checks[check_name] = {'error': e, 'traceback': traceback_message}
                continue
        else:
            log.debug('No conf.d/%s.yaml found for checks_d/%s.py' % (check_name, check_name))
            continue

        # Look for the per-check config, which *must* exist
        if 'instances' not in check_config:
            log.error("Config %s is missing 'instances'" % conf_path)
            continue

        # Init all of the check's classes with
        init_config = check_config.get('init_config', {})
        # init_config: in the configuration triggers init_config to be defined
        # to None.
        if init_config is None:
            init_config = {}

        instances = check_config['instances']
        try:
            try:
                c = check_class(check_name, init_config=init_config,
                                agent_config=agent_config, instances=instances)
            except TypeError as e:
                # Backwards compatibility for checks which don't support the
                # instances argument in the constructor.
                c = check_class(check_name, init_config=init_config,
                                agent_config=agent_config)
                c.instances = instances
        except Exception as e:
            log.exception('Unable to initialize check %s' % check_name)
            traceback_message = traceback.format_exc()
            init_failed_checks[check_name] = {'error': e, 'traceback': traceback_message}
        else:
            initialized_checks[check_name] = c

        # Add custom pythonpath(s) if available
        if 'pythonpath' in check_config:
            pythonpath = check_config['pythonpath']
            if not isinstance(pythonpath, list):
                pythonpath = [pythonpath]
            sys.path.extend(pythonpath)

        log.debug('Loaded check.d/%s.py' % check_name)

    log.info('Successfully initialized checks: %s' % initialized_checks.keys())
    if len(init_failed_checks) > 0:
        log.info('Initialization failed for checks: %s' % init_failed_checks.keys())
    return {'initialized_checks': initialized_checks.values(),
            'init_failed_checks': init_failed_checks,
            }


def initialize_logging(logger_name):
    try:
        log_format = '%%(asctime)s | %%(levelname)s | %s | %%(name)s(%%(filename)s:%%(lineno)s) | %%(message)s' % logger_name
        log_date_format = "%Y-%m-%d %H:%M:%S %Z"
        config = configuration.Config()
        logging_config = config.get_config(sections='Logging')

        logging.basicConfig(
            format=log_format,
            level=logging_config['log_level'] or logging.INFO,
        )

        # set up file loggers
        log_file = logging_config.get('%s_log_file' % logger_name)
        if log_file is not None and not logging_config['disable_file_logging']:
            # make sure the log directory is writable
            # NOTE: the entire directory needs to be writable so that rotation works
            if os.access(os.path.dirname(log_file), os.R_OK | os.W_OK):
                file_handler = logging.handlers.RotatingFileHandler(
                    log_file, maxBytes=LOGGING_MAX_BYTES, backupCount=1)
                formatter = logging.Formatter(log_format, log_date_format)
                file_handler.setFormatter(formatter)

                root_log = logging.getLogger()
                root_log.addHandler(file_handler)
            else:
                sys.stderr.write("Log file is unwritable: '%s'\n" % log_file)

        # set up syslog
        if logging_config['log_to_syslog']:
            try:
                syslog_format = '%s[%%(process)d]: %%(levelname)s (%%(filename)s:%%(lineno)s): %%(message)s' % logger_name
                from logging.handlers import SysLogHandler

                if logging_config['syslog_host'] is not None and logging_config[
                        'syslog_port'] is not None:
                    sys_log_addr = (logging_config['syslog_host'], logging_config['syslog_port'])
                else:
                    sys_log_addr = "/dev/log"
                    # Special-case macs
                    if sys.platform == 'darwin':
                        sys_log_addr = "/var/run/syslog"

                handler = SysLogHandler(address=sys_log_addr, facility=SysLogHandler.LOG_DAEMON)
                handler.setFormatter(
                    logging.Formatter(syslog_format, log_date_format))
                root_log = logging.getLogger()
                root_log.addHandler(handler)
            except Exception as e:
                sys.stderr.write("Error setting up syslog: '%s'\n" % str(e))
                traceback.print_exc()

    except Exception as e:
        sys.stderr.write("Couldn't initialize logging: %s\n" % str(e))
        traceback.print_exc()

        # if config fails entirely, enable basic stdout logging as a fallback
        logging.basicConfig(
            format=log_format,
            level=logging.INFO,
        )

    # re-get the log after logging is initialized
    global log
    log = logging.getLogger(__name__)


"""
Iterable Recipes
"""


def chunks(iterable, chunk_size):
    """Generate sequences of `chunk_size` elements from `iterable`."""
    iterable = iter(iterable)
    while True:
        chunk = [None] * chunk_size
        count = 0
        try:
            for _ in range(chunk_size):
                chunk[count] = iterable.next()
                count += 1
            yield chunk[:count]
        except StopIteration:
            if count:
                yield chunk[:count]
            break


def get_sub_collection_warn():
    config = configuration.Config()
    agent_config = config.get_config(sections='Main')
    sub_collection_warn = agent_config.get('sub_collection_warn')
    return sub_collection_warn


def get_collector_restart_interval():
    config = configuration.Config()
    agent_config = config.get_config(sections='Main')
    restart_interval = agent_config.get('collector_restart_interval')
    restart_interval_in_sec = restart_interval * 60 * 60
    return restart_interval_in_sec



import os
import shutil
import unittest
import uuid

from common import get_check


class TestCrash(unittest.TestCase):

    def setUp(self):
        self.crash_dir = '/tmp/crash-test-%s' % str(uuid.uuid4())
        os.mkdir(self.crash_dir)

    def tearDown(self):
        shutil.rmtree(self.crash_dir)

    def test_checks(self):
        config = """
init_config:
   crash_dir: %s

instances:
   - name: crash_stats
""" % self.crash_dir

        (check, instances) = get_check('crash', config)

        # Baseline check
        check.check(instances[0])
        metrics = check.get_metrics()
        self.assertEqual(metrics[0].value, 0)
        self.assertEqual(metrics[0].value_meta['latest'], '')

        # Add a crash and re-check
        os.mkdir(os.path.join(self.crash_dir,'201504141011'))

        check.check(instances[0])
        metrics = check.get_metrics()
        self.assertEqual(metrics[0].value, 1)
        self.assertEqual(metrics[0].value_meta['latest'],
                         '2015-04-14 10:11:00')

        # Add a second crash and re-check
        os.mkdir(os.path.join(self.crash_dir,'201505222303'))

        check.check(instances[0])
        metrics = check.get_metrics()
        self.assertEqual(metrics[0].value, 2)
        self.assertEqual(metrics[0].value_meta['latest'],
                         '2015-05-22 23:03:00')


if __name__ == "__main__":
    unittest.main()



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import json
import logging
import os
import subprocess

import monasca_agent.collector.checks as checks

log = logging.getLogger(__name__)

DIAG_COMMAND = '/usr/local/bin/diagnostics'
CHECKER_COMMAND = '/usr/local/bin/swift-checker'

DIAG_ATTRS = ['check_mounts', 'disk_monitoring', 'file_ownership',
              'network_interface', 'ping_hosts', 'drive_audit']
CHECKER_ATTRS = ['diskusage', 'healthcheck', 'replication']


def run_command(command, input=None):
    log.info("Executing command - {0}".format(command))
    try:
        process = subprocess.Popen(command,
                                   shell=True,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
                                   stdin=subprocess.PIPE)
        stdout, stderr = process.communicate(input=input)
        errcode = process.returncode
        log.debug('errcode - {0}, stdout - {1}, stderr - {2}'.format(errcode,
                                                                     stdout,
                                                                     stderr))
        return errcode, stdout, stderr
    except Exception:
        log.error("Failure while executing command - {0}".format(command))


def process_command(command):
    """Runs the command and returns json output
    """
    try:
        errcode, stdout, stderr = run_command(command)
        json_output = json.loads(stdout)
        return json_output
    except Exception:
        log.error('Failure while processing output - {0}'.format(stdout))


class SwiftDiags(checks.AgentCheck):
    def __init__(self, name, init_config, agent_config):
        super(SwiftDiags, self).__init__(name, init_config, agent_config)

    def check(self, instance):
        """Get swift checks and propagate.
           The checks are part of HP swift-diags package and checks are
           are run only if the package exists.
        """
        if not (os.path.exists(DIAG_COMMAND) and
                os.path.exists(CHECKER_COMMAND)):
            return None

        dimensions = self._set_dimensions(None, instance)

        for attribute in DIAG_ATTRS:
            # have to give sudo privileges for these commands to execute
            cmd = '{0} {1} --{2}'.format('sudo', DIAG_COMMAND, attribute)
            try:
                output = process_command(cmd)
                self.gauge('swift.{0}'.format(attribute),
                           int(output['status']),
                           dimensions=dimensions,
                           value_meta={'message': output.get('message')})
            except Exception as e:
                log.error('Error in performing check - {0}, {1}'.
                          format(attribute, e.message))

        for attribute in CHECKER_ATTRS:
            # have to give sudo privileges for these commands to execute
            cmd = '{0} {1} --{2}'.format('sudo', CHECKER_COMMAND, attribute)
            try:
                output = process_command(cmd)
                self.gauge('swift.{0}'.format(attribute),
                           int(output['status']), dimensions=dimensions,
                           value_meta={'message': output.get('message')})
                if output.get('data') and 'name' in output.get('data', {}):
                    metric = 'swift.{0}.{1}'.format(attribute,
                                                    output['data']['name'])
                    value = output['data']['value']
                    self.gauge(metric, value=value, dimensions=dimensions)
            except Exception as e:
                log.error('Error in performing check - {0}, {1}'.
                          format(attribute, e.message))



# (C) Copyright 2015,2016 Hewlett Packard Enterprise Development LP
"""Base class for Checks.

If you are writing your own checks you should subclass the AgentCheck class.
The Check class is being deprecated so don't write new checks with it.
"""
# This file uses 'print' as a function rather than a statement, a la Python3
from __future__ import print_function

import logging
import os
import re

import yaml

import monasca_agent.common.aggregator as aggregator
import monasca_agent.common.metrics as metrics_pkg
import monasca_agent.common.util as util


class AgentCheck(util.Dimensions):

    def __init__(self, name, init_config, agent_config, instances=None):
        """Initialize a new check.

        :param name: The name of the check
        :param init_config: The config for initializing the check
        :param agent_config: The global configuration for the agent
        :param instances: A list of configuration objects for each instance.
        """
        super(AgentCheck, self).__init__(agent_config)
        self.name = name
        self.init_config = init_config
        self.hostname = util.get_hostname()
        self.log = logging.getLogger('%s.%s' % (__name__, name))

        threshold = agent_config.get('recent_point_threshold', None)
        tenant_id = agent_config.get('global_delegated_tenant', None)
        self.aggregator = (
            aggregator.MetricsAggregator(self.hostname,
                                         recent_point_threshold=threshold,
                                         tenant_id=tenant_id))

        self.instances = instances or []
        self.library_versions = None

    def instance_count(self):
        """Return the number of instances that are configured for this check.
        """
        return len(self.instances)

    def gauge(self, metric, value, dimensions=None, delegated_tenant=None, hostname=None,
              device_name=None, timestamp=None, value_meta=None):
        """Record the value of a gauge, with optional dimensions, hostname, value metadata and device name.

        :param metric: The name of the metric
        :param value: The value of the gauge
        :param dimensions: (optional) A dictionary of dimensions for this metric
        :param delegated_tenant: (optional) Submit metrics on behalf of this tenant ID.
        :param hostname: (optional) A hostname for this metric. Defaults to the current hostname.
        :param device_name: (optional) The device name for this metric
        :param timestamp: (optional) The timestamp for this metric value
        :param value_meta: Additional metadata about this value
        """
        self.aggregator.submit_metric(metric,
                                      value,
                                      metrics_pkg.Gauge,
                                      dimensions,
                                      delegated_tenant,
                                      hostname,
                                      device_name,
                                      value_meta,
                                      timestamp)

    def increment(self, metric, value=1, dimensions=None, delegated_tenant=None,
                  hostname=None, device_name=None, value_meta=None):
        """Increment a counter with optional dimensions, hostname and device name.

        :param metric: The name of the metric
        :param value: The value to increment by
        :param dimensions: (optional) A dictionary of dimensions for this metric
        :param delegated_tenant: (optional) Submit metrics on behalf of this tenant ID.
        :param hostname: (optional) A hostname for this metric. Defaults to the current hostname.
        :param device_name: (optional) The device name for this metric
        :param value_meta: Additional metadata about this value
        """
        self.aggregator.submit_metric(metric,
                                      value,
                                      metrics_pkg.Counter,
                                      dimensions,
                                      delegated_tenant,
                                      hostname,
                                      device_name,
                                      value_meta)

    def decrement(self, metric, value=1, dimensions=None, delegated_tenant=None,
                  hostname=None, device_name=None, value_meta=None):
        """Decrement a counter with optional dimensions, hostname and device name.

        :param metric: The name of the metric
        :param value: The value to decrement by
        :param dimensions: (optional) A dictionary of dimensions for this metric
        :param delegated_tenant: (optional) Submit metrics on behalf of this tenant ID.
        :param hostname: (optional) A hostname for this metric. Defaults to the current hostname.
        :param device_name: (optional) The device name for this metric
        :param value_meta: Additional metadata about this value
        """
        value *= -1
        self.aggregator.submit_metric(metric,
                                      value,
                                      metrics_pkg.Counter,
                                      dimensions,
                                      delegated_tenant,
                                      hostname,
                                      device_name,
                                      value_meta)

    def rate(self, metric, value, dimensions=None, delegated_tenant=None,
             hostname=None, device_name=None, value_meta=None):
        """Submit a point for a metric that will be calculated as a rate on flush.

        Values will persist across each call to `check` if there is not enough
        point to generate a rate on the flush.

        :param metric: The name of the metric
        :param value: The value of the rate
        :param dimensions: (optional) A dictionary of dimensions for this metric
        :param delegated_tenant: (optional) Submit metrics on behalf of this tenant ID.
        :param hostname: (optional) A hostname for this metric. Defaults to the current hostname.
        :param device_name: (optional) The device name for this metric
        :param value_meta: Additional metadata about this value
        """
        self.aggregator.submit_metric(metric,
                                      value,
                                      metrics_pkg.Rate,
                                      dimensions,
                                      delegated_tenant,
                                      hostname,
                                      device_name,
                                      value_meta)

    def get_metrics(self, prettyprint=False):
        """Get all metrics, including the ones that are tagged.

        @return the list of samples
        @rtype list of Measurement objects from monasca_agent.common.metrics
        """
        metrics = self.aggregator.flush()
        if prettyprint:
            for metric in metrics:
                measurement = metric['measurement']
                print(" Timestamp: {0}".format(measurement['timestamp']))
                print(" Name:       {0}".format(measurement['name']))
                print(" Value:      {0}".format(measurement['value']))
                print(" Dimensions: ", end='')
                line = 0
                dimensions = measurement['dimensions']
                for name in dimensions:
                    if line != 0:
                        print(" " * 13, end='')
                    print("{0}={1}".format(name, dimensions[name]))
                    line += 1

                print(" Value Meta: ", end='')
                value_meta = measurement['value_meta']
                if value_meta:
                    line = 0
                    for name in value_meta:
                        if line != 0:
                            print(" " * 13, end='')
                        print("{0}={1}".format(name, value_meta[name]))
                        line += 1
                else:
                    print('None')
                print("-" * 24)

        return metrics

    def get_library_info(self):
        if self.library_versions is not None:
            return self.library_versions
        try:
            self.library_versions = self.get_library_versions()
        except NotImplementedError:
            pass

    def get_library_versions(self):
        """Should return a string that shows which version

        of the needed libraries are used
        """
        raise NotImplementedError

    def prepare_run(self):
        """Do any setup required before running all instances"""
        return

    def run(self):
        """Run all instances.
        """
        self.prepare_run()

        for i, instance in enumerate(self.instances):
            try:
                self.check(instance)
            except Exception:
                self.log.exception("Check '%s' instance #%s failed" % (self.name, i))

    def check(self, instance):
        """Overriden by the check class. This will be called to run the check.

        :param instance: A dict with the instance information. This will vary
        depending on your config structure.
        """
        raise NotImplementedError()

    @staticmethod
    def stop():
        """To be executed when the agent is being stopped to clean resources.
        """
        pass

    @classmethod
    def from_yaml(cls, path_to_yaml=None, agentConfig=None, yaml_text=None, check_name=None):
        """A method used for testing your check without running the agent.
        """

        if path_to_yaml:
            check_name = os.path.basename(path_to_yaml).split('.')[0]
            try:
                f = open(path_to_yaml)
            except IOError:
                raise Exception('Unable to open yaml config: %s' % path_to_yaml)
            yaml_text = f.read()
            f.close()

        config = yaml.safe_load(yaml_text)
        check = cls(check_name, config.get('init_config') or {}, agentConfig or {})

        return check, config.get('instances', [])

    @staticmethod
    def normalize(metric, prefix=None):
        """Turn a metric into a well-formed metric name prefix.b.c

        :param metric The metric name to normalize
        :param prefix A prefix to to add to the normalized name, default None
        """
        name = re.sub(r"[,\+\*\-/()\[\]{}]", "_", metric)
        # Eliminate multiple _
        name = re.sub(r"__+", "_", name)
        # Don't start/end with _
        name = re.sub(r"^_", "", name)
        name = re.sub(r"_$", "", name)
        # Drop ._ and _.
        name = re.sub(r"\._", ".", name)
        name = re.sub(r"_\.", ".", name)

        if prefix is not None:
            return prefix + "." + name
        else:
            return name

    @staticmethod
    def read_config(instance, key, message=None, cast=None, optional=False):
        val = instance.get(key)
        if val is None:
            if optional is False:
                message = message or 'Must provide `%s` value in instance config' % key
                raise Exception(message)
            else:
                return val

        if cast is None:
            return val
        else:
            return cast(val)



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import logging
import os

from monasca_setup import agent_config
from monasca_setup.detection import Plugin

log = logging.getLogger(__name__)


class Crash(Plugin):
    """Detect if kdump is installed and enabled and setup configuration to
       monitor for crash dumps.
    """

    def _detect(self):
        """Run detection, set self.available True if a crash kernel is loaded.
        """
        loaded = '/sys/kernel/kexec_crash_loaded'
        if os.path.isfile(loaded):
            with open(loaded, 'r') as fh:
                if fh.read().strip() == '1':
                    self.available = True

    def build_config(self):
        """Build the config as a Plugin object and return it.
        """
        log.info('\tEnabling the Monasca crash dump healthcheck')
        config = agent_config.Plugins()

        config['crash'] = {
            'init_config': None,
            'instances': [{'name': 'crash_stats'}]
        }

        return config

    def dependencies_installed(self):
        return True



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import monasca_setup.detection


class Ovsvapp(monasca_setup.detection.ServicePlugin):

    """Detect OVSvApp service VM specific daemons and setup configuration to monitor them.

    """

    def __init__(self, template_dir, overwrite=True, args=None):
        service_params = {
            'args': args,
            'template_dir': template_dir,
            'overwrite': overwrite,
            'service_name': 'OVSvApp-ServiceVM',
            'process_names': ['neutron-ovsvapp-agent', 'ovsdb-server', 'ovs-vswitchd'],
            'service_api_url': '',
            'search_pattern': ''
        }

        super(Ovsvapp, self).__init__(service_params)

    def build_config(self):
        """Build the config as a Plugins object and return."""
        self.service_api_url = None
        self.search_pattern = None

        return monasca_setup.detection.ServicePlugin.build_config(self)



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import re
import subprocess
import xml.parsers.expat  # python 2.4 compatible

from monasca_agent.collector.checks import AgentCheck


class Varnish(AgentCheck):
    # XML parsing bits, a.k.a. Kafka in Code

    def _reset(self):
        self._current_element = ""
        self._current_metric = "varnish"
        self._current_value = 0
        self._current_str = ""
        self._current_type = ""

    def _start_element(self, name, attrs):
        self._current_element = name

    def _end_element(self, name):
        if name == "stat":
            m_name = self.normalize(self._current_metric)
            if self._current_type in ("a", "c"):
                self.rate(m_name, long(self._current_value))
            elif self._current_type in ("i", "g"):
                self.gauge(m_name, long(self._current_value))
            else:
                # Unsupported data type, ignore
                self._reset()
                return  # don't save

            # reset for next stat element
            self._reset()
        elif name in ("type", "ident", "name"):
            self._current_metric += "." + self._current_str

    def _char_data(self, data):
        self.log.debug("Data %s [%s]" % (data, self._current_element))
        data = data.strip()
        if len(data) > 0 and self._current_element != "":
            if self._current_element == "value":
                self._current_value = long(data)
            elif self._current_element == "flag":
                self._current_type = data
            else:
                self._current_str = data

    def check(self, instance):
        """Extract stats from varnishstat -x

        The text option (-1) is not reliable enough when counters get large.
        VBE.media_video_prd_services_01(10.93.67.16,,8080).happy18446744073709551615

        2 types of data, "a" for counter ("c" in newer versions of varnish), "i" for gauge ("g")
        https://github.com/varnish/Varnish-Cache/blob/master/include/tbl/vsc_fields.h

        Bitmaps are not supported.

        <varnishstat>
            <stat>
                <name>fetch_304</name>
                <value>0</value>
                <flag>a</flag>
                <description>Fetch no body (304)</description>
            </stat>
            <stat>
                <name>n_sess_mem</name>
                <value>334</value>
                <flag>i</flag>
                <description>N struct sess_mem</description>
            </stat>
            <stat>
                <type>LCK</type>
                <ident>vcl</ident>
                <name>creat</name>
                <value>1</value>
                <flag>a</flag>
                <description>Created locks</description>
            </stat>
        </varnishstat>
        """
        # Not configured? Not a problem.
        if instance.get("varnishstat", None) is None:
            raise Exception("varnishstat is not configured")
        dimensions = self._set_dimensions(None, instance)
        name = instance.get('name')

        # Get the varnish version from varnishstat
        output, error = subprocess.Popen([instance.get("varnishstat"), "-V"],
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE).communicate()

        # Assumptions regarding varnish's version
        use_xml = True
        arg = "-x"  # varnishstat argument
        version = 3

        m1 = re.search(r"varnish-(\d+)", output, re.MULTILINE)
        # v2 prints the version on stderr, v3 on stdout
        m2 = re.search(r"varnish-(\d+)", error, re.MULTILINE)

        if m1 is None and m2 is None:
            self.log.warn("Cannot determine the version of varnishstat, assuming 3 or greater")
            self.log.warn("Cannot determine the version of varnishstat, assuming 3 or greater")
        else:
            if m1 is not None:
                version = int(m1.group(1))
            elif m2 is not None:
                version = int(m2.group(1))

        self.log.debug("Varnish version: %d" % version)

        # Location of varnishstat
        if version <= 2:
            use_xml = False
            arg = "-1"

        cmd = [instance.get("varnishstat"), arg]
        if name is not None:
            cmd.extend(['-n', name])
            dimensions.update({'varnish_name': name})
        else:
            dimensions.update({'varnish_name': 'default'})
        try:
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
            output, error = proc.communicate()
        except Exception:
            self.log.error(u"Failed to run %s" % repr(cmd))
            raise
        if error and len(error) > 0:
            self.log.error(error)
        self._parse_varnishstat(output, use_xml, dimensions)

    def _parse_varnishstat(self, output, use_xml, dimensions):
        if use_xml:
            p = xml.parsers.expat.ParserCreate()
            p.StartElementHandler = self._start_element
            p.EndElementHandler = self._end_element
            p.CharacterDataHandler = self._char_data
            self._reset()
            p.Parse(output, True)
        else:
            for line in output.split("\n"):
                self.log.debug("Parsing varnish results: %s" % line)
                fields = line.split()
                if len(fields) < 3:
                    break
                name, gauge_val, rate_val = fields[0], fields[1], fields[2]
                metric_name = self.normalize(name, prefix="varnish")

                # Now figure out which value to pick
                if rate_val.lower() in ("nan", "."):
                    # col 2 matters
                    self.log.debug("Varnish (gauge) %s %d" % (metric_name, int(gauge_val)))
                    self.gauge(metric_name, int(gauge_val), dimensions=dimensions)
                else:
                    # col 3 has a rate (since restart)
                    self.log.debug("Varnish (rate) %s %d" % (metric_name, int(gauge_val)))
                    self.rate(metric_name, float(gauge_val), dimensions=dimensions)



# (C) Copyright 2016 Hewlett Packard Enterprise Development Company LP

import logging

from monasca_agent.common.util import timeout_command
import monasca_setup.agent_config
import monasca_setup.detection
from monasca_setup.detection.utils import find_process_name
from monasca_setup.detection.utils import watch_process_by_username

log = logging.getLogger(__name__)

VERTICA_SERVICE = 'vertica'
CONNECTION_TIMEOUT = 3
SERVICE = 'vertica'
USER = 'monitor'
USER_PASSWORD = 'password'


class Vertica(monasca_setup.detection.Plugin):

    """Detect Vertica process running and DB connection status

        This plugin has the following options (each optional) that you can pass in via command line:
            - user      (optional - user to connect with) - Defaults to monitor user
            - password  (optional - password to use when connecting) - Defaults to password
            - service   (optional - dimensions service to be set for the metrics coming out of the plugin)
            - timeout   (optional - timeout for vertica connection in seconds) - Defaults to 3 second
    """

    def _detect(self):
        """Run detection, set self.available True if the service is detected.

        """
        if (find_process_name('vertica') is not None and find_process_name(
                'spread') is not None):
            self.available = True

    def _get_config(self):
        """Set the configuration to be used for connecting to Vertica
        :return:
        """
        # Set defaults and read config or use arguments
        if self.args is None:
            self.user = USER
            self.password = USER_PASSWORD
            self.service = VERTICA_SERVICE
            self.timeout = CONNECTION_TIMEOUT
        else:
            self.user = self.args.get('user', USER)
            self.password = self.args.get('password', USER_PASSWORD)
            self.service = self.args.get('service', VERTICA_SERVICE)
            self.timeout = int(self.args.get('timeout', CONNECTION_TIMEOUT))

    def _connection_test(self):
        """Attempt to connect to Vertica DB to verify credentials.
        :return: bool status of the test
        """
        log.info("\tVertica connection test.")
        stdout, stderr, return_code = timeout_command(
            ["/opt/vertica/bin/vsql", "-U", self.user, "-w", self.password, "-t", "-A", "-c",
             "SELECT node_name FROM current_session"], self.timeout)
        # remove trailing newline
        stdout = stdout.rstrip()
        if return_code == 0:
            self.node_name = stdout
            return True
        else:
            log.error("Error querying vertica with return code of {0} and the error {1}".format(return_code, stderr))
            return False

    def build_config(self):
        """Build the config as a Plugins object and return.

        """
        config = monasca_setup.agent_config.Plugins()
        try:
            self._get_config()
            config.merge(watch_process_by_username('dbadmin', 'vertica', self.service, 'vertica'))
            log.info("\tWatching the vertica processes.")
            if self._connection_test():
                log.info("\tBuilding vertica config.")
                instance_config = {'name': 'localhost',
                                   'user': self.user,
                                   'password': self.password,
                                   'service': self.service,
                                   'node_name': self.node_name,
                                   'timeout': self.timeout}
                config['vertica'] = {'init_config': None, 'instances': [instance_config]}
            else:
                exception_msg = 'Unable to connect to the Vertica DB. ' \
                                'The Vertica plugin is not configured. ' \
                                'Please correct and re-run monasca-setup.'
                log.error(exception_msg)
                raise Exception(exception_msg)
        except Exception as e:
            exception_msg = 'Error configuring the Vertica check plugin - {0}'.format(e)
            log.error(exception_msg)
            raise Exception(exception_msg)

        return config



# (C) Copyright 2015,2016 Hewlett Packard Enterprise Development LP

from __future__ import absolute_import

import os
import re

import docker

from monasca_agent.collector import checks

CONTAINER_ID_RE = re.compile('[0-9a-f]{64}')
DEFAULT_BASE_URL = "unix://var/run/docker.sock"
DEFAULT_VERSION = "auto"
DEFAULT_TIMEOUT = 3
DEFAULT_ADD_KUBERNETES_DIMENSIONS = False
JIFFY_HZ = os.sysconf(os.sysconf_names['SC_CLK_TCK'])
CGROUPS = ['cpuacct', 'memory', 'blkio']


class Docker(checks.AgentCheck):
    """Collect metrics and events from Docker API and cgroups"""

    def __init__(self, name, init_config, agent_config, instances=None):
        checks.AgentCheck.__init__(self, name, init_config, agent_config, instances)

        if instances is not None and len(instances) > 1:
            raise Exception('Docker check only supports one configured instance.')

        self.connection_timeout = int(init_config.get('connection_timeout', DEFAULT_TIMEOUT))
        self.docker_version = init_config.get('version', DEFAULT_VERSION)
        self.docker_root = init_config.get('docker_root', '/')
        # Locate cgroups directories
        self._mount_points = {}
        self._cgroup_filename_pattern = None
        for cgroup in CGROUPS:
            self._mount_points[cgroup] = self._find_cgroup(cgroup)
        self._prev_cpu = {}
        self._curr_cpu = {}
        self._cpu_count = None
        self._prev_system_cpu = None

    def check(self, instance):
        docker_url = instance.get('url', DEFAULT_BASE_URL)
        try:
            docker_client = docker.Client(base_url=docker_url, version=self.docker_version,
                                          timeout=self.connection_timeout)
            running_containers = {container['Id']: container for container in self._get_containers(docker_client)}
        except Exception as e:
            self.log.error("Could not get containers from Docker API skipping Docker check - {}".format(e))
            return
        add_kubernetes_dimensions = instance.get('add_kubernetes_dimensions', DEFAULT_ADD_KUBERNETES_DIMENSIONS)
        dimensions = self._set_dimensions(None, instance)
        self.gauge("container.running_count", len(running_containers), dimensions=dimensions)
        self._set_container_pids(running_containers)
        # Report container metrics from cgroups
        self._report_container_metrics(running_containers, add_kubernetes_dimensions, dimensions)

    def _report_rate_gauge_metric(self, metric_name, value, dimensions):
        self.rate(metric_name + "_sec", value, dimensions=dimensions)
        self.gauge(metric_name, value, dimensions=dimensions)

    def _report_container_metrics(self, container_dict, add_kubernetes_dimensions, dimensions):
        self._curr_system_cpu, self._cpu_count = self._get_system_cpu_ns()
        system_memory = self._get_total_memory()
        for container in container_dict.itervalues():
            try:
                container_dimensions = dimensions.copy()
                container_id = container['Id']
                container['name'] = self._get_container_name(container['Names'], container_id)
                container_dimensions['image'] = container['Image']
                container_labels = container['Labels']
                if add_kubernetes_dimensions:
                    if 'io.kubernetes.pod.name' in container_labels:
                        container_dimensions['kubernetes_pod_name'] = container_labels['io.kubernetes.pod.name']
                    if 'io.kubernetes.pod.namespace' in container_labels:
                        container_dimensions['kubernetes_namespace'] = container_labels['io.kubernetes.pod.namespace']
                self._report_cgroup_cpuacct(container_id, container_dimensions)
                self._report_cgroup_memory(container_id, container_dimensions, system_memory)
                self._report_cgroup_blkio(container_id, container_dimensions)
                if "_proc_root" in container:
                    self._report_net_metrics(container, container_dimensions)

                self._report_cgroup_cpu_pct(container_id, container_dimensions)
            except IOError as err:
                # It is possible that the container got stopped between the
                # API call and now
                self.log.info("IO error while collecting cgroup metrics, "
                              "skipping container...", exc_info=err)
            except Exception as err:
                self.log.error("Error when collecting data about container {}".format(err))
        self._prev_system_cpu = self._curr_system_cpu

    def _get_container_name(self, container_names, container_id):
        container_name = None
        if container_names:
            for name in container_names:
                # if there is more than one / the name is actually an alias
                if name.count('/') <= 1:
                    container_name = str(name).lstrip('/')
                    break
        return container_name if container_name else container_id

    def _report_cgroup_cpuacct(self, container_id, container_dimensions):
        stat_file = self._get_cgroup_file('cpuacct', container_id, 'cpuacct.stat')
        stats = self._parse_cgroup_pairs(stat_file)
        self._report_rate_gauge_metric('container.cpu.user_time', stats['user'], container_dimensions)
        self._report_rate_gauge_metric('container.cpu.system_time', stats['system'], container_dimensions)

    def _report_cgroup_memory(self, container_id, container_dimensions, system_memory_limit):
        stat_file = self._get_cgroup_file('memory', container_id, 'memory.stat')
        stats = self._parse_cgroup_pairs(stat_file)

        cache_memory = stats['cache']
        rss_memory = stats['rss']
        self.gauge('container.mem.cache', cache_memory, dimensions=container_dimensions)
        self.gauge('container.mem.rss', rss_memory, dimensions=container_dimensions)

        swap_memory = 0
        if 'swap' in stats:
            swap_memory = stats['swap']
            self.gauge('container.mem.swap', swap_memory, dimensions=container_dimensions)

        # Get container max memory
        memory_limit_file = self._get_cgroup_file('memory', container_id, 'memory.limit_in_bytes')
        memory_limit = self._parse_cgroup_value(memory_limit_file, convert=float)
        if memory_limit > system_memory_limit:
            memory_limit = float(system_memory_limit)
        used_perc = round((((cache_memory + rss_memory + swap_memory) / memory_limit) * 100), 2)
        self.gauge('container.mem.used_perc', used_perc, dimensions=container_dimensions)

    def _report_cgroup_blkio(self, container_id, container_dimensions):
        stat_file = self._get_cgroup_file('blkio', container_id,
                                          'blkio.throttle.io_service_bytes')
        stats = self._parse_cgroup_blkio_metrics(stat_file)
        self._report_rate_gauge_metric('container.io.read_bytes', stats['io_read'], container_dimensions)
        self._report_rate_gauge_metric('container.io.write_bytes', stats['io_write'], container_dimensions)

    def _report_cgroup_cpu_pct(self, container_id, container_dimensions):
        usage_file = self._get_cgroup_file('cpuacct', container_id, 'cpuacct.usage')

        prev_cpu = self._prev_cpu.get(container_id, None)
        curr_cpu = self._parse_cgroup_value(usage_file)
        self._prev_cpu[container_id] = curr_cpu

        if prev_cpu is None:
            # probably first run, we need 2 data points
            return

        system_cpu_delta = float(self._curr_system_cpu - self._prev_system_cpu)
        container_cpu_delta = float(curr_cpu - prev_cpu)
        if system_cpu_delta > 0 and container_cpu_delta > 0:
            cpu_pct = (container_cpu_delta / system_cpu_delta) * self._cpu_count * 100
            self.gauge('container.cpu.utilization_perc', cpu_pct, dimensions=container_dimensions)

    def _report_net_metrics(self, container, container_dimensions):
        """Find container network metrics by looking at /proc/$PID/net/dev of the container process."""
        proc_net_file = os.path.join(container['_proc_root'], 'net/dev')
        try:
            with open(proc_net_file, 'r') as f:
                lines = f.readlines()
                """Two first lines are headers:
                Inter-|   Receive                                                |  Transmit
                 face |bytes    packets errs drop fifo frame compressed multicast|bytes    packets errs drop fifo colls carrier compressed
                """
                for line in lines[2:]:
                    cols = line.split(':', 1)
                    interface_name = str(cols[0]).strip()
                    if interface_name != 'lo':
                        container_network_dimensions = container_dimensions.copy()
                        container_network_dimensions['interface'] = interface_name
                        network_values = cols[1].split()
                        self._report_rate_gauge_metric("container.net.in_bytes", long(network_values[0]),
                                                       container_network_dimensions)
                        self._report_rate_gauge_metric("container.net.out_bytes", long(network_values[8]),
                                                       container_network_dimensions)
                        break
        except Exception as e:
            self.log.error("Failed to report network metrics from file {0}. Exception: {1}".format(proc_net_file, e))

    # Docker API
    def _get_containers(self, docker_client):
        """Gets the list of running containers in Docker."""
        return docker_client.containers()

    def _find_cgroup_filename_pattern(self, container_id):
        # We try with different cgroups so that it works even if only one is properly working
        for mountpoint in self._mount_points.itervalues():
            stat_file_path_lxc = os.path.join(mountpoint, "lxc")
            stat_file_path_docker = os.path.join(mountpoint, "docker")
            stat_file_path_coreos = os.path.join(mountpoint, "system.slice")
            stat_file_path_kubernetes = os.path.join(mountpoint, container_id)
            stat_file_path_kubernetes_docker = os.path.join(mountpoint, "system", "docker", container_id)
            stat_file_path_docker_daemon = os.path.join(mountpoint, "docker-daemon", "docker", container_id)

            if os.path.exists(stat_file_path_lxc):
                return '%(mountpoint)s/lxc/%(id)s/%(file)s'
            elif os.path.exists(stat_file_path_docker):
                return '%(mountpoint)s/docker/%(id)s/%(file)s'
            elif os.path.exists(stat_file_path_coreos):
                return '%(mountpoint)s/system.slice/docker-%(id)s.scope/%(file)s'
            elif os.path.exists(stat_file_path_kubernetes):
                return '%(mountpoint)s/%(id)s/%(file)s'
            elif os.path.exists(stat_file_path_kubernetes_docker):
                return '%(mountpoint)s/system/docker/%(id)s/%(file)s'
            elif os.path.exists(stat_file_path_docker_daemon):
                return '%(mountpoint)s/docker-daemon/docker/%(id)s/%(file)s'

        raise Exception("Cannot find Docker cgroup directory. Be sure your system is supported.")

    def _get_cgroup_file(self, cgroup, container_id, filename):
        # This can't be initialized at startup because cgroups may not be mounted yet
        if not self._cgroup_filename_pattern:
            self._cgroup_filename_pattern = self._find_cgroup_filename_pattern(container_id)

        return self._cgroup_filename_pattern % (dict(
            mountpoint=self._mount_points[cgroup],
            id=container_id,
            file=filename,
        ))

    def _get_total_memory(self):
        with open(os.path.join(self.docker_root, '/proc/meminfo')) as f:
            for line in f.readlines():
                tokens = line.split()
                if tokens[0] == 'MemTotal:':
                    return int(tokens[1]) * 1024

        raise Exception('Invalid formatting in /proc/meminfo: unable to '
                        'determine MemTotal')

    def _get_system_cpu_ns(self):
        # see also: getSystemCPUUsage of docker's stats_collector_unix.go
        total_jiffies = None
        cpu_count = 0

        with open(os.path.join(self.docker_root, '/proc/stat'), 'r') as f:
            for line in f.readlines():
                tokens = line.split()

                if tokens[0] == 'cpu':
                    if len(tokens) < 8:
                        raise Exception("Invalid formatting in /proc/stat")

                    total_jiffies = sum(map(lambda t: int(t), tokens[1:8]))
                elif tokens[0].startswith('cpu'):
                    # startswith but does not equal implies /cpu\d+/ or so
                    # we don't need full per-cpu usage to calculate %,
                    # so just count cores
                    cpu_count += 1

        if not total_jiffies:
            raise Exception("Unable to find CPU usage in /proc/stat")

        cpu_time_ns = (total_jiffies / JIFFY_HZ) * 1e9
        return cpu_time_ns, cpu_count

    def _find_cgroup(self, hierarchy):
        """Finds the mount point for a specified cgroup hierarchy. Works with
        old style and new style mounts.
        """
        with open(os.path.join(self.docker_root, "/proc/mounts"), 'r') as f:
            mounts = map(lambda x: x.split(), f.read().splitlines())

        cgroup_mounts = filter(lambda x: x[2] == "cgroup", mounts)
        if len(cgroup_mounts) == 0:
            raise Exception("Can't find mounted cgroups. If you run the Agent inside a container,"
                            " please refer to the documentation.")
        # Old cgroup style
        if len(cgroup_mounts) == 1:
            return os.path.join(self.docker_root, cgroup_mounts[0][1])

        candidate = None
        for _, mountpoint, _, opts, _, _ in cgroup_mounts:
            if hierarchy in opts:
                if mountpoint.startswith("/host/"):
                    return os.path.join(self.docker_root, mountpoint)
                candidate = mountpoint
        if candidate is not None:
            return os.path.join(self.docker_root, candidate)
        raise Exception("Can't find mounted %s cgroups." % hierarchy)

    def _parse_cgroup_value(self, stat_file, convert=int):
        """Parse a cgroup info file containing a single value."""
        with open(stat_file, 'r') as f:
            return convert(f.read().strip())

    def _parse_cgroup_pairs(self, stat_file, convert=int):
        """Parse a cgroup file for key/values."""
        with open(stat_file, 'r') as f:
            split_lines = map(lambda x: x.split(' ', 1), f.readlines())
            return {k: convert(v) for k, v in split_lines}

    def _parse_cgroup_blkio_metrics(self, stat_file):
        """Parse the blkio metrics."""
        with open(stat_file, 'r') as f:
            stats = f.read().splitlines()
            metrics = {
                'io_read': 0,
                'io_write': 0,
            }
            for line in stats:
                if 'Read' in line:
                    metrics['io_read'] += int(line.split()[2])
                if 'Write' in line:
                    metrics['io_write'] += int(line.split()[2])
            return metrics

    # checking if cgroup is a container cgroup
    def _is_container_cgroup(self, line, selinux_policy):
        if line[1] not in ('cpu,cpuacct', 'cpuacct,cpu', 'cpuacct') or line[2] == '/docker-daemon':
            return False
        if 'docker' in line[2]:
            return True
        if 'docker' in selinux_policy:
            return True
        if line[2].startswith('/') and re.match(CONTAINER_ID_RE, line[2][1:]):  # kubernetes
            return True
        return False

    def _set_container_pids(self, containers):
        """Find all proc paths for running containers."""
        proc_path = os.path.join(self.docker_root, 'proc')
        pid_dirs = [_dir for _dir in os.listdir(proc_path) if _dir.isdigit()]

        for pid_dir in pid_dirs:
            try:
                path = os.path.join(proc_path, pid_dir, 'cgroup')
                with open(path, 'r') as f:
                    content = [line.strip().split(':') for line in f.readlines()]

                selinux_policy = ''
                path = os.path.join(proc_path, pid_dir, 'attr', 'current')
                if os.path.exists(path):
                    with open(path, 'r') as f:
                        selinux_policy = f.readlines()[0]
            except IOError as e:
                self.log.debug("Cannot read %s, "
                               "process likely raced to finish : %s" %
                               (path, str(e)))
                continue
            except Exception as e:
                self.log.warning("Cannot read %s : %s" % (path, str(e)))
                continue

            try:
                cpuacct = None
                for line in content:
                    if self._is_container_cgroup(line, selinux_policy):
                        cpuacct = line[2]
                        break
                matches = re.findall(CONTAINER_ID_RE, cpuacct) if cpuacct else None
                if matches:
                    container_id = matches[-1]
                    if container_id not in containers:
                        self.log.debug("Container %s not in container_dict, it's likely excluded", container_id)
                        continue
                    containers[container_id]['_pid'] = pid_dir
                    containers[container_id]['_proc_root'] = os.path.join(proc_path, pid_dir)
            except Exception as e:
                self.log.warning("Cannot parse %s content: %s" % (path, str(e)))
                continue



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

"""Classes for detection of running resources to be monitored.

    Detection classes should be platform independent
"""
import ast
import logging
import sys

log = logging.getLogger(__name__)


class Plugin(object):
    """Abstract class implemented by the monasca-agent plugin detection classes. """

    def __init__(self, template_dir, overwrite=True, args=None):
        self.available = False
        self.template_dir = template_dir
        self.dependencies = ()
        self.overwrite = overwrite
        if args is not None and isinstance(args, str):
            try:
                # Turn 'hostname=host type=ping' to dictionary {'hostname': 'host', 'type': 'ping'}
                self.args = dict([a.split('=') for a in args.split()])
            except Exception:
                log.exception('Error parsing detection arguments')
                sys.exit(1)
        elif isinstance(args, dict):
            self.args = args
        else:
            self.args = None
        self._detect()

    def _detect(self):
        """Run detection, set self.available True if the service is detected.

        """
        raise NotImplementedError

    def build_config(self):
        """Build the config as a Plugins object and return.

        """
        raise NotImplementedError

    def build_config_with_name(self):
        """Builds the config and then adds a field 'built_by' to each instance in the config.
           built_by is set to the plugin name
        :return: An agent_config.Plugins object
        """
        conf = self.build_config()
        if conf is None:
            return None
        for plugin_type in conf.itervalues():
            for inst in plugin_type['instances']:
                inst['built_by'] = self.__class__.__name__
        return conf

    def dependencies_installed(self):
        """Return True if dependencies are installed.

        """
        raise NotImplementedError

    def literal_eval(self, testval):
        """Return a literal boolean value if applicable

        """
        if 'false' in str(testval).lower() or 'true' in str(testval).lower():
            return ast.literal_eval(str(testval).capitalize())
        else:
            return testval

    @property
    def name(self):
        """Return _name if set otherwise the class name.

        """
        if '_name' in self.__dict__:
            return self._name
        else:
            return self.__class__.__name__



import unittest
import subprocess
import time
import urllib2
import tempfile
import os
import logging

from tests.common import load_check, kill_subprocess
from nose.plugins.attrib import attr
from nose.plugins.skip import SkipTest

logging.basicConfig()

MAX_WAIT = 30
HAPROXY_CFG = os.path.realpath(os.path.join(os.path.dirname(__file__), "haproxy.cfg"))
HAPROXY_OPEN_CFG = os.path.realpath(os.path.join(os.path.dirname(__file__), "haproxy-open.cfg"))


class HaproxyTestCase(unittest.TestCase):

    def _wait(self, url):
        loop = 0
        while True:
            try:
                STATS_URL = ";csv;norefresh"
                passman = urllib2.HTTPPasswordMgrWithDefaultRealm()
                passman.add_password(None, url, "datadog", "isdevops")
                authhandler = urllib2.HTTPBasicAuthHandler(passman)
                opener = urllib2.build_opener(authhandler)
                urllib2.install_opener(opener)
                url = "%s%s" % (url, STATS_URL)
                req = urllib2.Request(url)
                request = urllib2.urlopen(req)
                break
            except Exception:
                time.sleep(0.5)
                loop += 1
                if loop >= MAX_WAIT:
                    break
        return request

    def setUp(self):
        self.process = None
        self.cfg = None

    def start_server(self, config_fn, config):
        self.agent_config = {
            'version': '0.1',
            'api_key': 'toto'
        }

        # Initialize the check from checks_d
        self.check = load_check('haproxy', config, self.agent_config)

        try:
            self.cfg = tempfile.NamedTemporaryFile()
            self.cfg.write(open(config_fn).read())
            self.cfg.flush()
            # Start haproxy
            self.process = subprocess.Popen(["haproxy", "-d", "-f", self.cfg.name],
                                            executable="haproxy",
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)

            # Wait for it to really start
            self._wait("http://localhost:3834/stats")
        except Exception:
            logging.getLogger().exception("Cannot instantiate haproxy")

    @attr('haproxy')
    def testCheck(self):
        raise SkipTest('Requires Haproxy')
        config = {
            'init_config': {},
            'instances': [{
                'url': 'http://localhost:3834/stats',
                'username': 'datadog',
                'password': 'isdevops',
                'status_check': True,
                'collect_aggregates_only': False,
            }]
        }
        self.start_server(HAPROXY_CFG, config)

        # Run the check against our running server
        self.check.check(config['instances'][0])
        # Sleep for 1 second so the rate interval >=1
        time.sleep(1)
        # Run the check again so we get the rates
        self.check.check(config['instances'][0])

        # Metric assertions
        metrics = self.check.get_metrics()
        assert metrics
        self.assertIsInstance(metrics, list)
        self.assertTrue(len(metrics) > 0)

        self.assertEqual(len([t for t in metrics
                              if t[0] == "haproxy.backend.bytes.in_rate"]), 4, metrics)
        self.assertEqual(len([t for t in metrics
                              if t[0] == "haproxy.frontend.session.current"]), 1, metrics)

        inst = config['instances'][0]
        data = self.check._fetch_data(inst['url'], inst['username'], inst['password'])
        new_data = [l.replace("OPEN", "DOWN") for l in data]
        self.check._process_data(new_data, False, True, inst['url']),

        assert self.check.has_events()
        assert len(self.check.get_events()) == 1

    @attr('haproxy')
    def testWrongConfig(self):
        # Same check, with wrong data
        config = {
            'init_config': {},
            'instances': [{
                'url': 'http://localhost:3834/stats',
                'username': 'wrong',
                'password': 'isdevops',
                'collect_aggregates_only': False,
            }]
        }
        self.start_server(HAPROXY_CFG, config)

        # Run the check, make sure there are no metrics or events
        try:
            self.check.check(config['instances'][0])
        except Exception:
            pass
        else:
            assert False, "Should raise an error"
        metrics = self.check.get_metrics()
        assert len(metrics) == 0
        assert self.check.has_events() is False

    @attr('haproxy')
    def testOpenConfig(self):
        raise SkipTest('Requires Haproxy')
        # No passwords this time
        config = {
            'init_config': {},
            'instances': [{
                'url': 'http://localhost:3834/stats',
                'collect_aggregates_only': False,
            }]
        }
        self.start_server(HAPROXY_OPEN_CFG, config)

        # Run the check against our running server
        self.check.check(config['instances'][0])
        # Sleep for 1 second so the rate interval >=1
        time.sleep(1)
        # Run the check again so we get the rates
        self.check.check(config['instances'][0])

        metrics = self.check.get_metrics()
        assert metrics
        self.assertIsInstance(metrics, list)
        self.assertTrue(len(metrics) > 0)

        self.assertEqual(len([t for t in metrics
                              if t[0] == "haproxy.backend.bytes.in_rate"]), 4, metrics)
        self.assertEqual(len([t for t in metrics
                              if t[0] == "haproxy.frontend.session.current"]), 1, metrics)

    def tearDown(self):
        if self.process is not None:
            kill_subprocess(self.process)
        del self.cfg

if __name__ == "__main__":
    unittest.main()



#  Copyright 2017 Fujitsu LIMITED
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from collections import namedtuple

import psutil as psutil_orig


sconn = namedtuple('sconn', ['fd', 'family', 'type', 'laddr', 'raddr',
                             'status', 'pid'])

psutil = psutil_orig


def cpu_count(logical=True):
    if logical:
        return psutil.NUM_CPUS
    else:
        raise NotImplementedError


def net_connections(kind='inet'):
    ret = set()
    for p in psutil.process_iter():
        try:
            for c in p.connections(kind):
                conn = sconn(c.fd, c.family, c.type, c.laddr, c.raddr,
                             c.status, p.pid)
                ret.add(conn)
        except (psutil.AccessDenied, psutil.NoSuchProcess):
            pass
    return list(ret)


def children(self, recursive=False):
    return psutil.Process.get_children(self, recursive=recursive)


def connections(self, kind='inet'):
    return psutil.Process.get_connections(self, kind=kind)


def cpu_affinity(self, cpus=None):
    return psutil.Process.get_cpu_affinity(self)


def cpu_percent(self, interval=None):
    return psutil.Process.get_cpu_percent(self, interval=interval)


def cpu_times(self):
    return psutil.Process.get_cpu_times(self)


def memory_info_ex(self):
    return psutil.Process.get_ext_memory_info(self)


def io_counters(self):
    return psutil.Process.get_io_counters(self)


def ionice(self, ioclass=None, value=None):
    return psutil.Process.get_ionice(self)


def memory_info(self):
    return psutil.Process.get_memory_info(self)


def memory_maps(self, grouped=True):
    return psutil.Process.get_memory_maps(self, grouped=grouped)


def memory_percent(self):
    return psutil.Process.get_memory_percent(self)


def nice(self, value=None):
    return psutil.Process.get_nice(self)


def num_ctx_switches(self):
    return psutil.Process.get_num_ctx_switches(self)


def num_fds(self):
    return psutil.Process.get_num_fds(self)


def num_threads(self):
    return psutil.Process.get_num_threads(self)


def open_files(self):
    return psutil.Process.get_open_files(self)


def threads(self):
    return psutil.Process.get_threads(self)


def cwd(self):
    return psutil.Process.getcwd(self)


if psutil.version_info < (2, 0):
    psutil.cpu_count = cpu_count
    psutil.Process.children = children
    psutil.Process.connections = connections
    psutil.Process.cpu_affinity = cpu_affinity
    psutil.Process.cpu_percent = cpu_percent
    psutil.Process.cpu_times = cpu_times
    psutil.Process.memory_info_ex = memory_info_ex
    psutil.Process.io_counters = io_counters
    psutil.Process.ionice = ionice
    psutil.Process.memory_info = memory_info
    psutil.Process.memory_maps = memory_maps
    psutil.Process.memory_percent = memory_percent
    psutil.Process.nice = nice
    psutil.Process.num_ctx_switches = num_ctx_switches
    psutil.Process.num_fds = num_fds
    psutil.Process.num_threads = num_threads
    psutil.Process.open_files = open_files
    psutil.Process.threads = threads
    psutil.Process.cwd = cwd

if psutil.version_info < (2, 1):
    psutil.net_connections = net_connections



# (C) Copyright 2016 Hewlett Packard Enterprise Development LP
import unittest

import monasca_agent.common.metrics as metrics

SAMPLE_RATE = 1


class TestMetrics(unittest.TestCase):
    def test_Gauge(self):
        tenant_name = "test_gauge"
        metric_name = "foo"
        dimensions = {'a': 'b', 'c': 'd'}

        gauge = metrics.Gauge(metric_name, dimensions, tenant_name)

        gauge.sample(0, SAMPLE_RATE, 1)

        envelope = gauge.flush()[0]
        measurement = envelope['measurement']

        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 0)
        self.assertEqual(measurement['timestamp'], 1000)

        gauge.sample(1, SAMPLE_RATE, 2)

        envelope = gauge.flush()[0]
        measurement = envelope['measurement']

        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 1)
        self.assertEqual(measurement['timestamp'], 2000)

        gauge.sample(100.5212, SAMPLE_RATE, 125)

        envelope = gauge.flush()[0]
        measurement = envelope['measurement']

        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 100.5212)
        self.assertEqual(measurement['timestamp'], 125000)

        results = gauge.flush()
        self.assertEqual(results, [])

    def test_Counter(self):
        tenant_name = "test_counter"
        metric_name = "bar"
        dimensions = {'a': 1, 'c': 2}

        counter = metrics.Counter(metric_name, dimensions, tenant_name)

        counter.sample(5, SAMPLE_RATE, 1)

        envelope = counter.flush()[0]
        measurement = envelope['measurement']
        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 5)
        self.assertEqual(measurement['timestamp'], 1000)

        counter.sample(5, SAMPLE_RATE, 1)

        envelope = counter.flush()[0]
        measurement = envelope['measurement']
        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 5)
        self.assertEqual(measurement['timestamp'], 1000)

        counter.sample(5, SAMPLE_RATE, 1)
        counter.sample(5, SAMPLE_RATE, 1)

        envelope = counter.flush()[0]
        measurement = envelope['measurement']
        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 10)
        self.assertEqual(measurement['timestamp'], 1000)

        # Errors in counter report 0 value with previous timestamp
        counter.sample("WEGONI", SAMPLE_RATE, 2)
        envelope = counter.flush()[0]
        measurement = envelope['measurement']
        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 0)
        self.assertEqual(measurement['timestamp'], 1000)

    def test_Rate(self):
        tenant_name = "test_rate"
        metric_name = "baz"
        dimensions = {'a': 2, 'c': 3}

        rate = metrics.Rate(metric_name, dimensions, tenant_name)

        rate.sample(5, SAMPLE_RATE, 1)
        self.assertEqual(rate.flush(), [])

        rate.sample(5, SAMPLE_RATE, 2)

        envelope = rate.flush()[0]
        measurement = envelope['measurement']
        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 0.0)
        self.assertEqual(measurement['timestamp'], 2000)

        rate.sample(10, SAMPLE_RATE, 3)

        envelope = rate.flush()[0]
        measurement = envelope['measurement']
        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 5)
        self.assertEqual(measurement['timestamp'], 3000)

        rate.sample(12, SAMPLE_RATE, 3)
        self.assertEqual(rate.flush(), [])

        rate.sample(12, SAMPLE_RATE, 4)

        envelope = rate.flush()[0]
        measurement = envelope['measurement']
        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 0.0)
        self.assertEqual(measurement['timestamp'], 4000)

        rate.sample(14, SAMPLE_RATE, 5)

        envelope = rate.flush()[0]
        measurement = envelope['measurement']
        self.assertEqual(envelope['tenant_id'], tenant_name)
        self.assertEqual(measurement['name'], metric_name)
        self.assertEqual(measurement['dimensions'], dimensions)
        self.assertEqual(measurement['value'], 2)
        self.assertEqual(measurement['timestamp'], 5000)

        rate.sample(1, SAMPLE_RATE, 6)
        self.assertEqual(rate.flush(), [])



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import monasca_setup.detection


class Heat(monasca_setup.detection.ServicePlugin):

    """Detect Heat daemons and setup configuration to monitor them."""

    def __init__(self, template_dir, overwrite=True, args=None):
        service_params = {
            'args': args,
            'template_dir': template_dir,
            'overwrite': overwrite,
            'service_name': 'orchestration',
            'process_names': ['heat-api ', 'heat-api-cfn',
                              'heat-api-cloudwatch', 'heat-engine'],
            'service_api_url': 'http://localhost:8004',
            'search_pattern': '.*versions.*',
        }

        super(Heat, self).__init__(service_params)



# (C) Copyright 2016 Hewlett Packard Enterprise Development Company LP

import ConfigParser
import contextlib
import logging
import os
import psutil
import re
import unittest

from mock import patch
from mock.mock import MagicMock

from monasca_setup.detection.plugins.ovs import Ovs


LOG = logging.getLogger('monasca_setup.detection.plugins.ovs')


class ps_util_get_proc:
    cmdLine = ['/etc/neutron/neutron.conf']
    detect_warning = False
    def as_dict(self, attrs=None):
        return {'name': 'neutron-openvsw',
                'cmdline': self.cmdline()}

    def cmdline(self):
        if not ps_util_get_proc.detect_warning:
            return ['neutron-openvsw',
                    ps_util_get_proc.cmdLine[0]]
        else:
            return ['/opt/fake.txt']


class TestOvs(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)
        with patch.object(Ovs, '_detect') as mock_detect:
            self.ovs_obj = Ovs('temp_dir')
            self.has_option = [True, True, True, False, False, True]
            self.get_value = [MagicMock(), MagicMock(), MagicMock(),
                              'http://10.10.10.10', 'region1']
            self.assertTrue(mock_detect.called)

    def _detect(self, ovs_obj):
        ovs_obj.neutron_conf = None
        ovs_obj.available = False
        with contextlib.nested(
                patch.object(psutil, 'process_iter',
                             return_value=[ps_util_get_proc()]),
                patch.object(os.path, 'isfile', return_value=True),
                patch.object(ovs_obj, 'dependencies_installed',
                             return_value=True)) as (
                mock_process_iter, mock_isfile, dependencies):
            ovs_obj._detect()
            self.assertTrue(mock_process_iter.called)
            if not ps_util_get_proc.cmdLine:
                self.assertFalse(mock_isfile.called)

    def _build_config(self, ovs_obj, dependencies_installed=True):
        with patch.object(ConfigParser, 'SafeConfigParser') as mock_config_parser:
            config_parser_obj = mock_config_parser.return_value
            with contextlib.nested(
                    patch.object(LOG, 'info'),
                    patch.object(config_parser_obj, 'has_option',
                                 side_effect=self.has_option),
                    patch.object(config_parser_obj, 'get',
                                 side_effect=self.get_value)) as (
                         mock_log_info, mock_has_option, mock_get):
                result = ovs_obj.build_config()
                if dependencies_installed:
                    self.assertTrue(mock_log_info.called)
                    self.assertTrue(mock_has_option.called)
                    self.assertTrue(mock_get.called)
                    if not self.has_option[-1]:
                        self.assertIn(str(('nova', 'region_name')),
                                      str(mock_get.call_args_list[-1]))
                    else:
                        self.assertIn(str(('service_auth', 'region')),
                                      str(mock_get.call_args_list[-1]))
                return result

    def _build_config_with_arg(self, ovs_obj):
            result = self._build_config(ovs_obj)
            self.assertEqual(result['ovs']['init_config']['neutron_refresh'],
                             13000)
            self.assertFalse(result['ovs']['init_config']['network_use_bits'])
            self.assertIsInstance(result['ovs']['init_config']['admin_user'],
                                  MagicMock)
            self.assertIsInstance(result['ovs']['init_config']['admin_password'],
                                  MagicMock)
            self.assertIsInstance(result['ovs']['init_config']['admin_tenant_name'],
                                  MagicMock)
            self.assertEqual(result['ovs']['init_config']['identity_uri'],
                             'http://10.10.10.10/v2.0')
            self.assertEqual(result['ovs']['init_config']['region_name'],
                             'region1')
            self.assertEqual(result['ovs']['init_config']['cache_dir'],
                             "/dev/shm")
            self.assertEqual(result['ovs']['init_config']['ovs_cmd'],
                             "sudo /usr/bin/ovs-vsctl")
            self.assertFalse(result['ovs']['init_config']['use_absolute_metrics'])
            self.assertTrue(result['ovs']['init_config']['use_rate_metrics'])
            self.assertTrue(result['ovs']['init_config']['use_health_metrics'])
            return result

    def _build_config_without_args(self, ovs_obj):
        result = self._build_config(ovs_obj)
        self.assertEqual(result['ovs']['init_config']['neutron_refresh'],
                         14400)
        self.assertFalse(result['ovs']['init_config']['network_use_bits'])
        self.assertEqual(result['ovs']['init_config']['cache_dir'],
                         "/dev/shm")
        self.assertEqual(result['ovs']['init_config']['ovs_cmd'],
                         "sudo /usr/bin/ovs-vsctl")
        self.assertEqual(result['ovs']['init_config']['included_interface_re'],
                         'qg.*|vhu.*|sg.*')
        self.assertIsInstance(result['ovs']['init_config']['admin_user'],
                              MagicMock)
        self.assertIsInstance(result['ovs']['init_config']['admin_password'],
                              MagicMock)
        self.assertIsInstance(result['ovs']['init_config']['admin_tenant_name'],
                              MagicMock)
        self.assertTrue(result['ovs']['init_config']['use_absolute_metrics'])
        self.assertTrue(result['ovs']['init_config']['use_rate_metrics'])
        self.assertTrue(result['ovs']['init_config']['use_health_metrics'])
        return result

    def test_detect(self):
        self._detect(self.ovs_obj)
        self.assertTrue(self.ovs_obj.available)
        self.assertEqual(self.ovs_obj.neutron_conf,
                         '/etc/neutron/neutron.conf')

    def test_detect_devstack(self):
        ps_util_get_proc.cmdLine = ['--config-file=/opt/stack/neutron.conf']
        self._detect(self.ovs_obj)
        self.assertTrue(self.ovs_obj.available)
        self.assertEqual(self.ovs_obj.neutron_conf, '/opt/stack/neutron.conf')

    def test_detect_warning(self):
        with patch.object(LOG, 'error') as mock_log_warn:
            ps_util_get_proc.detect_warning = True
            self._detect(self.ovs_obj)
            self.assertFalse(self.ovs_obj.available)
            self.assertEqual(self.ovs_obj.neutron_conf, None)
            self.assertTrue(mock_log_warn.called)

    def test_detect_conf_file_path_given(self):
        self.ovs_obj.neutron_conf = None
        self.ovs_obj.args = {'conf_file_path': '/opt/stack/neutron.conf'}
        with contextlib.nested(
                patch.object(psutil, 'process_iter',
                             return_value=[ps_util_get_proc()]),
                patch.object(os.path, 'isfile', return_value=True),
                patch.object(self.ovs_obj, 'dependencies_installed',
                             return_value=True)) as (
                mock_process_iter, mock_isfile, dependencies):
            self.ovs_obj._detect()
            self.assertTrue(mock_isfile.called)
            self.assertTrue(self.ovs_obj.available)
            self.assertEqual(self.ovs_obj.neutron_conf,
                             '/opt/stack/neutron.conf')

    def test_build_config(self):
        self.ovs_obj.neutron_conf = 'neutron-conf'
        self._build_config_without_args(self.ovs_obj)

    def test_build_config_with_args(self):
        with patch.object(LOG, 'warn') as mock_log_warn:
            self.ovs_obj.neutron_conf = 'neutron-conf'
            self.ovs_obj.args = {'admin_user': 'admin',
                                 'admin_password': 'password',
                                 'admin_tenant_name': 'tenant',
                                 'identity_uri': '10.10.10.20',
                                 'region_name': 'region0',
                                 'neutron_refresh': 13000,
                                 'use_absolute_metrics': False}
            result = self._build_config_with_arg(self.ovs_obj)
            self.assertTrue(mock_log_warn.called)
            self.assertEqual(mock_log_warn.call_count, 5)
            self.assertEqual(result['ovs']['init_config']['included_interface_re'],
                             'qg.*|vhu.*|sg.*')

    def test_dependencies_not_installed(self):
        result = self.ovs_obj.dependencies_installed()
        self.assertEqual(result, False)

    def test_build_config_invalid_arg_warning(self):
        with patch.object(LOG, 'warn') as mock_log_warn:
            self.ovs_obj.neutron_conf = 'neutron-conf'
            self.ovs_obj.args = {'admin_user': 'admin',
                                 'admin_password': 'password',
                                 'admin_tenant_name': 'tenant',
                                 'identity_uri': '10.10.10.20',
                                 'region_name': 'region0',
                                 'neutron_refresh': 13000,
                                 'use_absolute_metrics': False,
                                 'invalid_arg': 'inv-arg'}
            result = self._build_config_with_arg(self.ovs_obj)
            self.assertTrue(mock_log_warn.called)
            self.assertEqual(mock_log_warn.call_count, 6)
            self.assertEqual(result['ovs']['init_config']['included_interface_re'],
                             'qg.*|vhu.*|sg.*')

    def test_build_config_if_auth_version(self):
        self.ovs_obj.neutron_conf = 'neutron-conf'
        self.has_option = [True, True, True, True, True, True]
        self.get_value = [MagicMock(), MagicMock(), MagicMock(),
                          'v3.0', 'http://10.10.10.10',
                          'http://10.10.10.10', 'region1']
        result = self._build_config_without_args(self.ovs_obj)
        self.assertEqual(result['ovs']['init_config']['identity_uri'],
                         'http://10.10.10.10/v3.0')

    def test_build_config_if_auth_url_has_version(self):
        self.ovs_obj.neutron_conf = 'neutron-conf'
        self.has_option = [True, True, True, True, True, True]
        self.get_value = [MagicMock(), MagicMock(), MagicMock(),
                          'v3.0', 'http://10.10.10.10/v1',
                          'http://10.10.10.10/v1', 'region1']
        result = self._build_config_without_args(self.ovs_obj)
        self.assertEqual(result['ovs']['init_config']['identity_uri'],
                         'http://10.10.10.10/v1/')

    def test_build_config_region_name_from_nova(self):
        self.ovs_obj.neutron_conf = 'neutron-conf'
        self.has_option = [True, True, True, False, False, False]
        self.get_value = [MagicMock(), MagicMock(), MagicMock(),
                          'http://10.10.10.10', 'region2']
        result = self._build_config_without_args(self.ovs_obj)
        self.assertEqual(result['ovs']['init_config']['identity_uri'],
                         'http://10.10.10.10/v2.0')
        self.assertEqual(result['ovs']['init_config']['region_name'],
                         'region2')

    def test_build_config_with_valid_interface_re(self):
        self.ovs_obj.neutron_conf = 'neutron-conf'
        self.ovs_obj.args = {'included_interface_re': 'tap.*',
                             'neutron_refresh': 13000,
                             'use_absolute_metrics': False}
        result = self._build_config_with_arg(self.ovs_obj)
        self.assertEqual(result['ovs']['init_config']['included_interface_re'],
                         'tap.*')

    def test_build_config_with_invalid_interface_re(self):
        self.ovs_obj.neutron_conf = 'neutron-conf'
        self.ovs_obj.args = {'included_interface_re': '[',
                             'neutron_refresh': 13000}
        with contextlib.nested(
                patch.object(re, 'compile', side_effect=re.error),
                patch.object(LOG, 'exception')) as (
                    mock_re_error, mock_log):
            self.assertRaises(Exception, self._build_config_with_arg, self.ovs_obj)
            self.assertTrue(mock_re_error.called)
            self.assertTrue(mock_log.called)



import unittest
import logging
from nose.plugins.attrib import attr
from nose.plugins.skip import SkipTest

from tests.common import get_check

logging.basicConfig()

CONFIG = """
init_config:

instances:
    -   host: .
        dimensions:
            dim1: value1
            dim2: value2
"""


class IISTestCase(unittest.TestCase):

    @attr('windows')
    def testIIS(self):
        raise SkipTest('Requires IIS and wmi')
        check, instances = get_check('iis', CONFIG)
        check.check(instances[0])
        metrics = check.get_metrics()

        base_metrics = [m[0] for m in check.METRICS]
        ret_metrics = [m[0] for m in metrics]
        ret_dimensions = [m[3]['dimensions'] for m in metrics]

        # Make sure each metric was captured
        for metric in base_metrics:
            assert metric in ret_metrics

        # Make sure everything is tagged correctly
        for dimensions in ret_dimensions:
            assert dimensions == {'dim1': 'value1', 'dim2': 'value2'}

if __name__ == "__main__":
    unittest.main()



# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development Company LP
"""Gather metrics on specific processes.

"""
from collections import defaultdict
from collections import namedtuple

import monasca_agent.collector.checks as checks
from monasca_agent.common.psutil_wrapper import psutil

ProcessStruct = namedtuple("Process", "name pid username cmdline")


class ProcessCheck(checks.AgentCheck):
    PROCESS_GAUGE = ('process.thread_count',
                     'process.cpu_perc',
                     'process.mem.rss_mbytes',
                     'process.open_file_descriptors',
                     'process.io.read_count',
                     'process.io.write_count',
                     'process.io.read_kbytes',
                     'process.io.write_kbytes')

    def __init__(self, name, init_config, agent_config, instances=None):
        super(ProcessCheck, self).__init__(name, init_config, agent_config,
                                           instances)
        self._cached_processes = defaultdict(dict)
        self._current_process_list = None

    def find_pids(self, search_string, username, exact_match=True):
        """Create a set of pids of selected processes.

        Search for search_string
        """
        found_process_list = []
        if username:
            found_process_list = \
                [proc.pid for proc in self._current_process_list if
                 proc.username == username]
        else:
            for string in search_string:
                if string == 'All':
                    found_process_list.extend(
                        [proc.pid for proc in self._current_process_list])
                elif exact_match:
                    found_process_list.extend(
                        [proc.pid for proc in self._current_process_list
                         if proc.name == string])
                else:
                    found_process_list.extend(
                        [proc.pid for proc in self._current_process_list
                         if string in proc.cmdline])

        return set(found_process_list)

    @staticmethod
    def _safely_increment_var(var, value):
        if var:
            return var + value
        else:
            return value

    def get_process_metrics(self, pids, name):
        processes_to_remove = set(self._cached_processes[name].keys()) - pids
        for pid in processes_to_remove:
            del self._cached_processes[name][pid]
        got_denied = False
        io_permission = True

        # initialize aggregation values
        total_thr = None
        total_cpu = None
        total_rss = None
        total_open_file_descriptors = None
        total_read_count = None
        total_write_count = None
        total_read_kbytes = None
        total_write_kbytes = None

        for pid in set(pids):
            try:
                added_process = False
                if pid not in self._cached_processes[name]:
                    p = psutil.Process(pid)
                    self._cached_processes[name][pid] = p
                    added_process = True
                else:
                    p = self._cached_processes[name][pid]

                mem = p.memory_info_ex()
                total_rss = self._safely_increment_var(total_rss, float(mem.rss / 1048576))
                total_thr = self._safely_increment_var(total_thr, p.num_threads())

                try:
                    total_open_file_descriptors = self._safely_increment_var(total_open_file_descriptors, float(p.num_fds()))
                except psutil.AccessDenied:
                    got_denied = True

                if not added_process:
                    total_cpu = self._safely_increment_var(total_cpu, p.cpu_percent(interval=None))
                else:
                    p.cpu_percent(interval=None)

                # user might not have permission to call io_counters()
                if io_permission:
                    try:
                        io_counters = p.io_counters()
                        total_read_count = self._safely_increment_var(total_read_count, io_counters.read_count)
                        total_write_count = self._safely_increment_var(total_write_count, io_counters.write_count)
                        total_read_kbytes = self._safely_increment_var(total_read_kbytes, float(io_counters.read_bytes / 1024))
                        total_write_kbytes = self._safely_increment_var(total_write_kbytes, float(io_counters.write_bytes / 1024))
                    except psutil.AccessDenied:
                        self.log.debug('monasca-agent user does not have ' +
                                       'access to I/O counters for process' +
                                       ' %d: %s'
                                       % (pid, p.as_dict(['name'])['name']))
                        io_permission = False
                        total_read_count = None
                        total_write_count = None
                        total_read_kbytes = None
                        total_write_kbytes = None

            # Skip processes dead in the meantime
            except psutil.NoSuchProcess:
                self.log.warn('Process %s disappeared while metrics were being collected' % pid)
                pass

        if got_denied:
            self.log.debug("The Monitoring Agent was denied access " +
                           "when trying to get the number of file descriptors")

        return dict(zip(ProcessCheck.PROCESS_GAUGE,
                        (total_thr, total_cpu, total_rss, total_open_file_descriptors, total_read_count,
                         total_write_count, total_read_kbytes, total_write_kbytes)))

    def prepare_run(self):
        """Collect the list of processes once before each run"""

        self._current_process_list = []

        for process in psutil.process_iter():
            try:
                process_dict = process.as_dict(
                    attrs=['name', 'pid', 'username', 'cmdline'])
                p = ProcessStruct(name=process_dict['name'],
                                  pid=process_dict['pid'],
                                  username=process_dict['username'],
                                  cmdline=' '.join(process_dict['cmdline']))
                self._current_process_list.append(p)
            except psutil.NoSuchProcess:
                # No way to log useful information here so just move on
                pass
            except psutil.AccessDenied as e:
                process_dict = process.as_dict(attrs=['name'])
                self.log.info('Access denied to process {0}: {1}'.format(
                    process_dict['name'], e))

    def check(self, instance):
        name = instance.get('name', None)
        exact_match = instance.get('exact_match', True)
        search_string = instance.get('search_string', None)
        username = instance.get('username', None)

        if name is None:
            raise KeyError('The "name" of process groups is mandatory')

        if (search_string is None and username is None) or (search_string is not None and username is not None):
            raise KeyError('"You must provide either "search_string" or "user"')

        if username is None:
            dimensions = self._set_dimensions({'process_name': name}, instance)
        else:
            dimensions = self._set_dimensions({'process_user': username, 'process_name': name}, instance)

        pids = self.find_pids(search_string, username, exact_match=exact_match)

        self.log.debug('ProcessCheck: process %s analysed' % name)

        self.gauge('process.pid_count', len(pids), dimensions=dimensions)

        if instance.get('detailed', False):
            metrics = self.get_process_metrics(pids, name)
            for metric_name, metric_value in metrics.iteritems():
                if metric_value is not None:
                    self.gauge(metric_name, metric_value, dimensions=dimensions)



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import monasca_setup.detection


class Cue(monasca_setup.detection.ServicePlugin):

    """Detect Cue daemons and setup configuration to monitor them."""

    def __init__(self, template_dir, overwrite=True, args=None):
        service_params = {
            'args': args,
            'template_dir': template_dir,
            'overwrite': overwrite,
            'service_name': 'message-broker',
            'process_names': ['cue-api', 'cue-worker', 'cue-monitor'],
            'service_api_url': 'http://localhost:8795/v1',
            'search_pattern': '.*v1.*'
        }

        super(Cue, self).__init__(service_params)
